# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2012 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Code for rpc message dispatching.

Messages that come in have a version number associated with them.  RPC API
version numbers are in the form:

    Major.Minor

For a given message with version X.Y, the receiver must be marked as able to
handle messages of version A.B, where:

    A = X

    B >= Y

The Major version number would be incremented for an almost completely new API.
The Minor version number would be incremented for backwards compatible changes
to an existing API.  A backwards compatible change could be something like
adding a new method, adding an argument to an existing method (but not
requiring it), or changing the type for an existing argument (but still
handling the old type as well).

The conversion over to a versioned API must be done on both the client side and
server side of the API at the same time.  However, as the code stands today,
there can be both versioned and unversioned APIs implemented in the same code
base.

EXAMPLES
========

Nova was the first project to use versioned rpc APIs.  Consider the compute rpc
API as an example.  The client side is in nova/compute/rpcapi.py and the server
side is in nova/compute/manager.py.

Example 1) Adding a new method.
-------------------------------

Adding a new method is a backwards compatible change.  It should be added to
nova/compute/manager.py, and RPC_API_VERSION should be bumped from X.Y to
X.Y+1.  On the client side, the new method in nova/compute/rpcapi.py should
have a specific version specified to indicate the minimum API version that must
be implemented for the method to be supported.  For example::

    def get_host_uptime(self, ctxt, host):
        topic = _compute_topic(self.topic, ctxt, host, None)
        return self.call(ctxt, self.make_msg('get_host_uptime'), topic,
                version='1.1')

In this case, version '1.1' is the first version that supported the
get_host_uptime() method.

Example 2) Adding a new parameter.
----------------------------------

Adding a new parameter to an rpc method can be made backwards compatible.  The
RPC_API_VERSION on the server side (nova/compute/manager.py) should be bumped.
The implementation of the method must not expect the parameter to be present.::

    def some_remote_method(self, arg1, arg2, newarg=None):
        # The code needs to deal with newarg=None for cases
        # where an older client sends a message without it.
        pass

On the client side, the same changes should be made as in example 1.  The
minimum version that supports the new parameter should be specified.
"""

from vsm.openstack.common.rpc import common as rpc_common

class RpcDispatcher(object):
    """Dispatch rpc messages according to the requested API version.

    This class can be used as the top level 'manager' for a service.  It
    contains a list of underlying managers that have an API_VERSION attribute.
    """

    def __init__(self, callbacks):
        """Initialize the rpc dispatcher.

        :param callbacks: List of proxy objects that are an instance
                          of a class with rpc methods exposed.  Each proxy
                          object should have an RPC_API_VERSION attribute.
        """
        self.callbacks = callbacks
        super(RpcDispatcher, self).__init__()

    def dispatch(self, ctxt, version, method, **kwargs):
        """Dispatch a message based on a requested version.

        :param ctxt: The request context
        :param version: The requested API version from the incoming message
        :param method: The method requested to be called by the incoming
                       message.
        :param kwargs: A dict of keyword arguments to be passed to the method.

        :returns: Whatever is returned by the underlying method that gets
                  called.
        """
        if not version:
            version = '1.0'

        had_compatible = False
        for proxyobj in self.callbacks:
            if hasattr(proxyobj, 'RPC_API_VERSION'):
                rpc_api_version = proxyobj.RPC_API_VERSION
            else:
                rpc_api_version = '1.0'
            is_compatible = rpc_common.version_is_compatible(rpc_api_version,
                                                             version)
            had_compatible = had_compatible or is_compatible
            if not hasattr(proxyobj, method):
                continue
            if is_compatible:
                return getattr(proxyobj, method)(ctxt, **kwargs)

        if had_compatible:
            raise AttributeError("No such RPC function '%s'" % method)
        else:
            raise rpc_common.UnsupportedRpcVersion(version=version)



# coding=utf-8

"""
Insert the collected values into a mysql table
"""

from Handler import Handler
import MySQLdb


class VSMMySQLHandler(Handler):
    """
    Implements the abstract Handler class, sending data to a mysql table
    """
    conn = None

    def __init__(self, config=None):
        """
        Create a new instance of the VSMMySQLHandler class
        """
        # Initialize Handler
        Handler.__init__(self, config)

        # Initialize Options
        self.hostname = self.config['hostname']
        self.port = int(self.config['port'])
        self.username = self.config['username']
        self.password = self.config['password']
        self.database = self.config['database']
        self.table = self.config['table']
        self.col_time = self.config['col_time']
        self.col_metric = self.config['col_metric']
        self.col_value = self.config['col_value']
        self.col_hostname = self.config['col_hostname']
        self.col_instance = self.config['col_instance']

        # Connect
        self._connect()

    def get_default_config_help(self):
        """
        Returns the help text for the configuration options for this handler
        """
        config = super(VSMMySQLHandler, self).get_default_config_help()

        config.update({
        })

        return config

    def get_default_config(self):
        """
        Return the default config for the handler
        """
        config = super(VSMMySQLHandler, self).get_default_config()

        config.update({
        })

        return config

    def __del__(self):
        """
        Destroy instance of the VSMMySQLHandler class
        """
        self._close()

    def process(self, metric):
        """
        Process a metric
        """
        # Just send the data
        self._send(str(metric))

    def _send(self, data):
        """
        Insert the data
        """
        data = data.strip().split(' ')
        data_name = data[0].split('.')
        if data_name[2] == 'cpu' and data_name[4] == 'idle' and data_name[3] != 'total' :
            try:
                cursor = self.conn.cursor()
                cursor.execute("INSERT INTO %s (%s, %s, %s, %s, %s) VALUES(%%s, %%s, %%s ,%%s, %%s)"
                               % (self.table, self.col_metric, self.col_hostname, self.col_instance,
                                  self.col_time, self.col_value),
                               ('_'.join(data_name[2:4]), data_name[1], data_name[4], data[2], data[1]))

                cursor.close()
                self.conn.commit()
            except BaseException, e:
                self.log.error("VSMMySQLHandler: Failed sending data. %s.", e)
                self._connect()
        elif data_name[2] == 'CephCollector':
            metric_name = '_'.join(data_name[6:])
            if metric_name in ['osd_op_r','osd_op_w','osd_op_rw','osd_op_in_bytes','osd_op_out_bytes','osd_op_rw_latency_avgcount','osd_op_r_latency_avgcount','osd_op_w_latency_avgcount','osd_op_rw_latency_sum','osd_op_r_latency_sum','osd_op_w_latency_sum']:
                try:
                    cursor = self.conn.cursor()
                    cursor.execute("INSERT INTO %s (%s, %s, %s, %s, %s) VALUES(%%s, %%s, %%s ,%%s, %%s)"
                                   % (self.table, self.col_metric, self.col_hostname, self.col_instance,
                                      self.col_time, self.col_value),
                                   ('_'.join(data_name[6:]), data_name[1], '_'.join(data_name[4:6]), data[2], data[1]))

                    cursor.close()
                    self.conn.commit()
                except BaseException, e:
                    # Log Error
                    self.log.error("VSMMySQLHandler: Failed sending data. %s.", e)
                    # Attempt to restablish connection
                    self._connect()


    def _connect(self):
        """
        Connect to the MySQL server
        """
        self._close()
        self.conn = MySQLdb.Connect(host=self.hostname,
                                    port=self.port,
                                    user=self.username,
                                    passwd=self.password,
                                    db=self.database)

    def _close(self):
        """
        Close the connection
        """
        if self.conn:
            self.conn.commit()
            self.conn.close()




# Copyright 2014 Intel Corporation, All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

"""
Physical Service
"""
import os
import json
from vsm import db
from vsm import flags
from vsm import context
from vsm import manager
from vsm.conductor import rpcapi as conductor_rpcapi
from vsm.openstack.common import log as logging
from vsm.openstack.common.periodic_task import periodic_task
from vsm.physical import driver

LOG = logging.getLogger(__name__)
FLAGS = flags.FLAGS

class PhysicalManager(manager.Manager):
    """PhysicalManager is mainly used to manage physical nodes."""

    RPC_API_VERSION = '1.2'

    def __init__(self, service_name=None, *args, **kwargs):
        super(PhysicalManager, self).__init__(*args, **kwargs)
        self.driver = driver.load_physical_driver()
        self._conductor_rpcapi = conductor_rpcapi.ConductorAPI()
        self._log_printed = False

    @periodic_task
    def update_disk_device(self, context):
        hostname = FLAGS.host
        device_list = self._conductor_rpcapi.\
                      device_get_by_hostname(context, hostname)
        if device_list:
            for device in device_list:
                values = {}
                if not os.path.exists(device['path']):
                    device_state = FLAGS.partition_status_missing
                else:
                    device_state = FLAGS.partition_status_ok
                values['state'] = device_state
                    #usage = self.driver.get_disk_usage(device['name'])
                    #device['total_capacity_kb'] = usage.total / 1024
                    #device['avail_capacity_kb'] = usage.free / 1024
                    #device['used_capacity_kb'] = usage.used / 1024
                if not os.path.exists(device['journal']):
                    journal_state = FLAGS.partition_status_missing
                else:
                    journal_state = FLAGS.partition_status_ok
                values['journal_state'] = journal_state

                db.device_update(context, device['id'], values)



#    Copyright 2014 Intel
#    All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Handles all requests to the conductor service."""

from oslo.config import cfg

from vsm.conductor import manager
from vsm.conductor import rpcapi
from vsm import exception as exc
from vsm.openstack.common import log as logging
from vsm.openstack.common.rpc import common as rpc_common
from vsm import utils

conductor_opts = [
    cfg.StrOpt('manager',
               default='vsm.conductor.manager.ConductorManager',
               help='full class name for the Manager for conductor'),
]
conductor_group = cfg.OptGroup(name='conductor',
                               title='Conductor Options')
CONF = cfg.CONF
CONF.register_group(conductor_group)
CONF.register_opts(conductor_opts, conductor_group)

LOG = logging.getLogger(__name__)

class API(object):
    """Conductor API that does updates via RPC to the ConductorManager."""

    def __init__(self):
        self.conductor_rpcapi = rpcapi.ConductorAPI()

    def get_osd_num(self, context, storage_group_id):
        return self.conductor_rpcapi.get_osd_num(context, storage_group_id)

    def list_storage_pool(self, context):
        return self.conductor_rpcapi.list_storage_pool(context)

    def get_storage_pool(self, context, id):
        return self.conductor_rpcapi.get_storage_pool(context, id)

    def get_storage_group_list(self, context):
        return self.conductor_rpcapi.get_storage_group_list(context)

    def get_server_list(self, context):
        return self.conductor_rpcapi.get_server_list(context)

    def get_server(self, context, id):
        return self.conductor_rpcapi.get_server(context, id)

    def add_servers(self, context, attrs):
        return self.conductor_rpcapi.add_servers(context, attrs)

    def get_cluster_list(self, context):
        return self.conductor_rpcapi.get_cluster_list(context)

    def create_cluster(self, context, attrs):
        return self.conductor_rpcapi.get_server_list(context, attrs)

    def get_zone_list(self, context):
        return self.conductor_rpcapi.get_zone_list(context)

    def create_zone(self, context, values):
        return self.conductor_rpcapi.create_zone(context, values)

    def get_mapping(self, context):
        return self.conductor_rpcapi.get_mapping(context)

    def test_service(self, context):
        return self.conductor_rpcapi.test_service(context)

    def check_poolname(self, context, poolname):
        return self.conductor_rpcapi.check_poolname(context, poolname)

    def create_storage_pool(self, context, body):
        return self.conductor_rpcapi.create_storage_pool(context, body)

    def get_ruleset_id(self, context, storage_group_id):
        return self.conductor_rpcapi.get_ruleset_id(context, storage_group_id)

    def count_hosts_by_storage_group_id(self, context, storage_group_id):
        return self.conductor_rpcapi.count_hosts_by_storage_group_id(context, \
                                     storage_group_id)

    #def get_server_list(self, context):
    #    return self.conductor_rpcapi.get_server_list(context)

    #def get_service_by_host_and_topic(self, context, host, topic):
    #    return self.conductor_rpcapi.get_service_by_host_and_topic(context, host, topic)
################################<ly

    #init_node
    def init_node_get_by_id_and_type(self, context, id, type):
        return self.conductor_rpcapi.\
               init_node_get_by_id_and_type(context, id, type)

    def init_node_get_by_id(self, context, id):
        return self.conductor_rpcapi.init_node_get_by_id(context, id)

    def init_node_create(self, context, values):
        return self.conductor_rpcapi.\
               init_node_create(context, values)

    def init_node_get_by_primary_public_ip(self, context, primary_public_ip):
        return self.conductor_rpcapi.\
               init_node_get_by_primary_public_ip(context, primary_public_ip)

    def init_node_get_by_secondary_public_ip(self, context, \
                                             secondary_public_ip):
        return self.conductor_rpcapi.\
               init_node_get_by_secondary_public_ip(context, \
               secondary_public_ip)

    def init_node_get_by_cluster_ip(self, context, cluster_ip):
        return self.conductor_rpcapi.\
               init_node_get_by_cluster_ip(context, cluster_ip)

    def init_node_update(self, context, id, values):
        return self.conductor_rpcapi.init_node_update(context, id, values)

    def init_node_update_status_by_id(self, context, init_node_id, status):
        """Update init nodes info."""
        #TODO delete this function in the futhure.
        #We will not expose this function in WSGI-api.
        return self.conductor_rpcapi.init_node_update_status_by_id(context,
                                                                   init_node_id,
                                                                   status)
    #osd_state
    def osd_get(self, context, osd_id):
        return self.conductor_rpcapi.\
               osd_get(context, osd_id)

    def osd_delete(self, context, osd_id):
        return self.conductor_rpcapi.\
               osd_delete(context, osd_id)

    def osd_remove(self, context, osd_id):
        return self.conductor_rpcapi.\
               osd_remove(context, osd_id)

    def osd_state_get_all(self, context, limit=None, marker=None, sort_keys=None, sort_dir=None,search_opts={}):
        return self.conductor_rpcapi.\
               osd_state_get_all(context, limit, marker, sort_keys, sort_dir, search_opts)

    def osd_state_update_or_create(self, context, values, create=None):
        return self.conductor_rpcapi.\
               osd_state_update_or_create(context, values, create)

    def osd_state_create(self, context, values):
        result = self.osd_state_get_by_osd_name_and_service_id_and_cluster_id(\
                 context, values['osd_name'], values['service_id'],\
                 values['cluster_id'])
        if not result:
            result = self.osd_state_get_by_device_id_and_service_id_and_cluster_id(\
                 context, values['device_id'], values['service_id'],\
                 values['cluster_id'])
        if not result:
            return self.conductor_rpcapi.\
               osd_state_update_or_create(context, values, create=True)
        else:
            values['id'] = result['id']
            values['deleted'] = 0
            return self.conductor_rpcapi.\
                osd_state_update_or_create(context, values, create=False)

    def osd_state_count_by_init_node_id(self, context, init_node_id):
        return self.conductor_rpcapi.\
               osd_state_count_by_init_node_id(context, init_node_id)

    def osd_state_get_by_service_id_and_storage_group_id(self, context, \
                                        service_id, storage_group_id):
        return self.conductor_rpcapi.\
               osd_state_get_by_service_id_and_storage_group_id(context, \
               service_id, storage_group_id)

    def osd_state_get_by_osd_name_and_service_id_and_cluster_id(\
        self, context, osd_name, service_id, cluster_id):
        return self.conductor_rpcapi.\
               osd_state_get_by_osd_name_and_service_id_and_cluster_id(\
               context, osd_name, service_id, cluster_id)
    def osd_state_get_by_device_id_and_service_id_and_cluster_id(\
        self, context, device_id, service_id, cluster_id):
        return self.conductor_rpcapi.\
               osd_state_get_by_device_id_and_service_id_and_cluster_id(\
               context, device_id, service_id, cluster_id)
    #device
    def device_get_all(self, context):
        return self.conductor_rpcapi.device_get_all(context)

    def device_get_all_by_service_id(self, context, service_id):
        return self.conductor_rpcapi.device_get_all_by_service_id(context, \
               service_id)

    def device_get_distinct_storage_class_by_service_id(self, context, \
                                                        service_id):
        return self.conductor_rpcapi.\
               device_get_distinct_storage_class_by_service_id(context, \
                                                        service_id)

    def device_create(self, context, values):
        return self.conductor_rpcapi.device_create(context, values)

    def device_update(self, context, device_id, values):
        values['id'] = device_id
        return self.conductor_rpcapi.\
               device_update_or_create(context, values, create=False)

    def device_get_by_name_and_journal_and_service_id(self, context, \
                                                  name, journal, \
                                                  service_id):
        return self.conductor_rpcapi.\
               device_get_by_name_and_journal_and_service_id(context, name, \
                                                journal, service_id)

    #zone
    def zone_get_all(self, context):
        return self.conductor_rpcapi.zone_get_all(context)

    def zone_get_by_id(self, context, id):
        return self.conductor_rpcapi.zone_get_by_id(context, id)

    def zone_get_by_name(self, context, name):
        return self.conductor_rpcapi.zone_get_by_name(context, name)

    #storage_group
    def storage_group_get_all(self, context):
        return self.conductor_rpcapi.storage_group_get_all(context)

    def storage_group_get(self, context, id):
        return self.conductor_rpcapi.storage_group_get(context, id)

    def create_storage_group(self, context, attrs):
        return self.conductor_rpcapi.create_storage_group(context, attrs)

    #cluster
    def cluster_create(self, context, values):
        return self.conductor_rpcapi.cluster_create(context, values)

    def cluster_get_by_name(self, context, name):
        return self.conductor_rpcapi.cluster_get_by_name(context, name)

    #service
    def service_get_by_host_and_topic(self, context, host, topic):
        return self.conductor_rpcapi.service_get_by_host_and_topic(context, \
                                     host, topic)
    #ceph
    def host_storage_groups_devices(self, context, \
                                    init_node_id):
        return self.conductor_rpcapi.\
               host_storage_groups_devices(context, \
               init_node_id)

    def ceph_node_info(self, context, init_node_id):
        return self.conductor_rpcapi.\
               ceph_node_info(context, init_node_id)

    #pg
    def pg_get_all(self, context, limit, marker, sort_keys, sort_dir):
        return self.conductor_rpcapi.\
               pg_get_all(context, limit, marker, sort_keys, sort_dir)

    def pg_get(self, context, id):
        return self.conductor_rpcapi.pg_get(context, id)

    #rbd
    def rbd_get_all(self, context, limit, marker, sort_keys, sort_dir):
        return self.conductor_rpcapi.\
               rbd_get_all(context, limit, marker, sort_keys, sort_dir)

    def rbd_get(self, context, id):
        return self.conductor_rpcapi.rbd_get(context, id)

    #license_status
    def license_status_create(self, context, values):
        return self.conductor_rpcapi.\
                license_status_create(context, values)

    def license_status_get(self, context):
        return self.conductor_rpcapi.license_status_get(context)

    def license_status_update(self, context, value):
        return self.conductor_rpcapi.\
               license_status_update(context, value)

    #mds
    def mds_get_all(self, context):
        return self.conductor_rpcapi.\
               mds_get_all(context)

    def mds_get(self, context, id):
        return self.conductor_rpcapi.mds_get(context, id)

    def ceph_error(self, context):
        return self.conductor_rpcapi.\
               ceph_error(context)

    def zones_hosts_get_by_storage_group(self, context, storage_group):
        return self.conductor_rpcapi.\
               zones_hosts_get_by_storage_group(context, storage_group)

    def get_performance_metrics(self, context, search_opts):
        return self.conductor_rpcapi.get_performance_metrics(context, search_opts)

    def get_sum_performance_metrics(self, context, search_opts):
        return self.conductor_rpcapi.get_sum_performance_metrics(context, search_opts)

    def get_latency(self, context, search_opts):
        return self.conductor_rpcapi.get_latency(context, search_opts)

    def get_cpu_usage(self, context, search_opts):
        return self.conductor_rpcapi.get_cpu_usage(context, search_opts)

    def get_poolusage(self, context, poolusage_id):
        return self.conductor_rpcapi.get_poolusage(context, poolusage_id)

    def get_appnode(self, context, id):
        return self.conductor_rpcapi.get_appnode(context, id)


#    Copyright 2014 Intel
#    All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Client side of the conductor RPC API."""
import logging
from oslo.config import cfg

from vsm.openstack.common import jsonutils
from vsm.openstack.common import rpc
import vsm.openstack.common.rpc.proxy

CONF = cfg.CONF

LOG = logging.getLogger(__name__)

class ConductorAPI(vsm.openstack.common.rpc.proxy.RpcProxy):
    """Client side of the conductor RPC API"""

    BASE_RPC_API_VERSION = '1.0'

    def __init__(self, topic=None):
        super(ConductorAPI, self).__init__(
            topic = topic or CONF.conductor_topic,
            default_version=self.BASE_RPC_API_VERSION)

    def test_service(self, ctxt):
        ret = self.call(ctxt, self.make_msg('test_service'))
        return ret

    def get_osd_num(self, ctxt, group_id):
        ret = self.call(ctxt, self.make_msg('get_osd_num', group_id=group_id))
        return ret

    def list_storage_pool(self, ctxt):
        ret = self.call(ctxt, self.make_msg('list_storage_pool'))
        return ret

    def get_storage_pool(self, ctxt, id):
        ret = self.call(ctxt, self.make_msg('get_storage_pool',
                                            id=id))
        return ret

    def destroy_storage_pool(self, ctxt, pool_name):
        self.cast(ctxt, self.make_msg('destroy_storage_pool', pool_name=pool_name))

    def update_storage_pool(self, context, pool_id, values):
        return self.call(context, self.make_msg('update_storage_pool',
                                                pool_id=pool_id, values=values))

    def update_storage_pool_by_name(self, context, pool_name, cluster_id, values):
        return self.call(context, self.make_msg('update_storage_pool_by_name',
                                                pool_name=pool_name,
                                                cluster_id=cluster_id,
                                                values=values))

    def get_storage_group_list(self, ctxt):
        ret = self.call(ctxt, self.make_msg('get_storage_group_list'))
        return ret

    def get_server_list(self, ctxt):
        ret = self.call(ctxt, self.make_msg('get_server_list'))
        return ret

    def get_server(self, ctxt, id):
        ret = self.call(ctxt, self.make_msg('get_server', id=id))
        return ret

    def add_servers(self, ctxt, attrs):
        ret = self.call(ctxt, self.make_msg('add_servers'), attrs=attrs)
        return ret

    def get_cluster_list(self, ctxt):
        ret = self.call(ctxt, self.make_msg('cluster_get_all'))
        return ret

    def create_cluster(self, ctxt, attrs):
        ret = self.call(ctxt, self.make_msg('create_cluster'), attrs=attrs)
        return ret

    def get_zone_list(self, ctxt):
        ret = self.call(ctxt, self.make_msg('get_zone_list'))
        return ret

    def create_zone(self, ctxt, values):
        ret = self.call(ctxt, self.make_msg('create_zone', values=values))
        return ret

    def get_mapping(self, ctxt):
        ret = self.call(ctxt, self.make_msg('get_mapping'))
        return ret

    def check_poolname(self, context, poolname):
        ret = self.call(context, self.make_msg('check_poolname', \
                        poolname=poolname))
        return ret

    def create_storage_pool(self, context, body):
        ret = self.call(context, self.make_msg('create_storage_pool', \
                        body=body))
        return ret

    def get_ruleset_id(self, ctxt, group_id):
        ret = self.call(ctxt, self.make_msg('get_ruleset_id', group_id=group_id))
        return ret

    def count_hosts_by_storage_group_id(self, context, storage_group_id):
        ret = self.call(context, self.make_msg('count_hosts_by_storage_group_id', \
                        storage_group_id=storage_group_id))
        return ret

#    def get_server_list(self, ctxt):
#        ret = self.call(ctxt, self.make_msg('get_server_list'))
#        return ret
#
#    def get_service_by_host_and_topic(self, ctxt, host, topic):
#        ret = self.call(ctxt, self.make_msg('get_service_by_host_and_topic', host=host, topic=topic))
#        return ret

#############################################ly>

    def init_node_get_by_host(self, ctxt, host):
        """Get init node ref by host name"""
        return self.call(ctxt,
                         self.make_msg('init_node_get_by_host',
                         host=host))

    def init_node_get_by_cluster_id(self, ctxt, cluster_id):
        """Get init node ref by host name"""
        return self.call(ctxt,
                         self.make_msg('init_node_get_by_cluster_id',
                         cluster_id=cluster_id))

    def init_node_get_cluster_nodes(self, ctxt, init_node_id):
        """Get nodes in the same cluster."""
        return self.call(ctxt,
                         self.make_msg('init_node_get_cluster_nodes',
                         init_node_id=init_node_id))

    #init_node
    def init_node_get_by_id_and_type(self, ctxt, id, type):
        ret = self.call(ctxt, self.make_msg('init_node_get_by_id_and_type',\
                        id=id, type=type))
        return ret

    def init_node_get_by_id(self, ctxt, id):
        ret = self.call(ctxt, self.make_msg('init_node_get_by_id', id=id))
        return ret

    def init_node_create(self, context, values):
        return  self.call(context, self.make_msg('init_node_create', \
                          values=values))

    def init_node_get_by_primary_public_ip(self, context, primary_public_ip):
        return self.call(context, self.\
                         make_msg('init_node_get_by_primary_public_ip',\
                         primary_public_ip=primary_public_ip))

    def init_node_get_by_secondary_public_ip(self, context, \
                                             secondary_public_ip):
        return self.call(context, self.\
                         make_msg('init_node_get_by_secondary_public_ip',\
                         secondary_public_ip=secondary_public_ip))

    def init_node_get_by_cluster_ip(self, context, cluster_ip):
        return self.call(context, self.make_msg('init_node_get_by_cluster_ip',\
                         cluster_ip=cluster_ip))

    def init_node_update(self, context, id, values):
        return self.call(context, self.make_msg('init_node_update', id=id, \
                         values=values))

    def init_node_update_status_by_id(self, context, init_node_id, status):
        """Update init node by id, change status."""
        return self.call(context,
                         self.make_msg('init_node_update_status_by_id',
                         init_node_id=init_node_id,
                         status=status))

    #osd_state
    def osd_get(self, context, osd_id):
        return self.call(context, self.make_msg('osd_get', osd_id=osd_id))

    def osd_delete(self, context, osd_id):
        return self.call(context, self.make_msg('osd_delete', osd_id=osd_id))

    def osd_remove(self, context, osd_id):
        return self.call(context, self.make_msg('osd_remove', osd_id=osd_id))

    def osd_state_get_all(self, context, limit, marker, sort_keys, sort_dir,search_opts):
        return self.call(context, self.make_msg('osd_state_get_all', \
                         limit=limit, marker=marker, sort_keys=sort_keys, \
                         sort_dir=sort_dir,search_opts=search_opts))

    def osd_state_create(self, context, values):
        return self.call(context, self.make_msg('osd_state_create', \
                         values=values))

    def osd_state_update(self, context, values):
        return self.call(context, self.make_msg('osd_state_update', \
                         values=values))

    def osd_state_update_or_create(self, context, values, create=None):
        return self.call(context, self.make_msg('osd_state_update_or_create', \
                         values=values, create=create), need_try=False)

    def osd_state_count_by_init_node_id(self, context, init_node_id):
        return self.call(context, self.\
                         make_msg('osd_state_count_by_init_node_id',\
                         init_node_id=init_node_id))

    def osd_state_get_by_service_id_and_storage_group_id(self, context, \
                                                         service_id, \
                                                         storage_group_id):
        return self.\
               call(context, \
               self.\
               make_msg('osd_state_get_by_service_id_and_storage_group_id', \
               service_id=service_id, storage_group_id=storage_group_id))

    def osd_state_get_by_service_id(self, context, service_id):
        return self.\
               call(context, \
               self.\
               make_msg('osd_state_get_by_service_id', \
               service_id=service_id))

    def osd_state_get_by_osd_name_and_service_id_and_cluster_id(self, \
        context, osd_name, service_id, cluster_id):
        return self.call(context, self.\
        make_msg('osd_state_get_by_osd_name_and_service_id_and_cluster_id',\
               osd_name=osd_name, service_id=service_id, \
               cluster_id=cluster_id))

    def osd_state_get_by_device_id_and_service_id_and_cluster_id(self, \
        context, device_id, service_id, cluster_id):
        return self.call(context, self.\
        make_msg('osd_state_get_by_device_id_and_service_id_and_cluster_id',\
               device_id=device_id, service_id=service_id, \
               cluster_id=cluster_id))

    #device
    def device_get_all(self, context):
        return self.call(context, self.make_msg('device_get_all'))

    def device_get_by_hostname(self, context, hostname):
        return self.call(context, self.make_msg('device_get_by_hostname', \
                         hostname=hostname))

    def device_create(self, context, values):
        return self.call(context, \
                         self.make_msg('device_create', values=values))

    def device_update_or_create(self, context, values, create=None):
        return self.call(context, self.make_msg('device_update_or_create', \
                         values=values, create=create))

    def device_get_all_by_service_id(self, context, service_id):
        return self.call(context, \
                         self.make_msg('device_get_all_by_service_id',\
                         service_id=service_id))

    def device_get_distinct_storage_class_by_service_id(self, context,\
                                                        service_id):
        return self.\
               call(context, self.\
               make_msg('device_get_distinct_storage_class_by_service_id',\
                        service_id=service_id))

    def device_get_by_name_and_journal_and_service_id(self, context, \
                                                  name, journal, \
                                                  service_id):
        return self.\
               call(context, self.\
               make_msg('device_get_by_name_and_journal_and_service_id', \
                        name=name, journal=journal, service_id=service_id))

    #storage_group
    def storage_group_get_all(self, context):
        return self.call(context, self.make_msg('storage_group_get_all'))

    def storage_group_get(self, context, id):
        return self.call(context, self.make_msg('storage_group_get',
                                                id=id))

    def create_storage_group(self, context, values):
        return self.call(context, self.make_msg('create_storage_group', \
                         values=values))

    #zone
    def zone_get_all(self, context):
        return self.call(context, self.make_msg('zone_get_all'))

    def zone_get_by_id(self, context, id):
        return self.call(context, self.make_msg('zone_get_by_id', id=id))

    def zone_get_by_name(self, context, name):
        return self.call(context, self.make_msg('zone_get_by_name',
                         name=name))

    #cluster
    def cluster_create(self, context, values):
        return self.call(context, self.make_msg('cluster_create', 
                                                values=values))

    def cluster_update(self, context, cluster_id, values):
        return self.call(context, self.make_msg('cluster_update',
                                                cluster_id=cluster_id,
                                                values=values))

    def cluster_get_by_name(self, context, name):
        return self.call(context, self.make_msg('cluster_get_by_name',
                                                name=name))

    def cluster_info_dict_get_by_id(self, context, cluster_id):
        return self.call(context,
                         self.make_msg('cluster_info_dict_get_by_id',
                                       cluster_id=cluster_id))

    #service
    def service_get_by_host_and_topic(self, context, host, topic):
        return self.call(context, \
                         self.make_msg('service_get_by_host_and_topic',
                         host=host, topic=topic))

    #ceph
    def host_storage_groups_devices(self, context, \
                                    init_node_id):
        return self.call(context, \
                         self.make_msg('host_storage_groups_devices',
                         init_node_id=init_node_id))

    def ceph_node_info(self, context, init_node_id):
        return self.call(context, \
                         self.make_msg('ceph_node_info', \
                         init_node_id=init_node_id))

    #pg
    def pg_get_all(self, context, limit, marker, sort_keys, sort_dir):
        return self.call(context, \
                         self.make_msg('pg_get_all',
                                       limit=limit, marker=marker,
                                       sort_keys=sort_keys,
                                       sort_dir=sort_dir))

    def pg_get(self, context, id):
        return self.call(context, self.make_msg('pg_get',
                                                id=id))

    #suggestion:you'd better not use this api! 
    #If the count of pgs is too large, it may blocks up the net.
    def pg_update_or_create(self, context, values):
        return self.call(context, \
                         self.make_msg('pg_update_or_create',
                         values=values))

    #rbd
    def rbd_get_all(self, context, limit, marker, sort_keys, sort_dir):
        return self.call(context, \
                         self.make_msg('rbd_get_all',
				       limit=limit, marker=marker,
					sort_keys=sort_keys, sort_dir=sort_dir))

    def rbd_get(self, context, id):
        return self.call(context, self.make_msg('rbd_get',
                                                id=id))

    #license_status
    def license_status_create(self, context, values):
        return self.call(context,
                         self.make_msg('license_status_create',
                                       values=values))

    def license_status_get(self, context):
        return self.call(context,
                         self.make_msg('license_status_get'))

    def license_status_update(self, context, value):
        return self.call(context,
                         self.make_msg('license_status_update',
                                       value=value))

    #mds
    def mds_get_all(self, context):
        return self.call(context, \
                         self.make_msg('mds_get_all'))

    def mds_get(self, context, id):
        return self.call(context,
                         self.make_msg('mds_get', id=id))

    def ceph_error(self, context):
        return self.call(context, \
                         self.make_msg('ceph_error'))

    def zones_hosts_get_by_storage_group(self, context, storage_group):
        return self.call(context, \
                         self.make_msg('zones_hosts_get_by_storage_group', 
                                        storage_group=storage_group))


    def get_performance_metrics(self, context, search_opts):
        return self.call(context,self.make_msg('get_performance_metrics', \
                               search_opts=search_opts))

    def get_sum_performance_metrics(self, context, search_opts):
        return self.call(context,self.make_msg('get_sum_performance_metrics', \
                               search_opts=search_opts))

    def get_latency(self, context, search_opts):
        return self.call(context,self.make_msg('get_latency', \
                               search_opts=search_opts))

    def get_cpu_usage(self, context, search_opts):
        return self.call(context,self.make_msg('get_cpu_usage', \
                               search_opts=search_opts))

    def get_poolusage(self, context, poolusage_id):
        return self.call(context, self.make_msg('get_poolusage',
                                                poolusage_id=poolusage_id))

    def get_appnode(self, context, appnode_id):
        return self.call(context, self.make_msg('get_appnode',
                                                appnode_id=appnode_id))

    def delete_pool_usage(self, context, poolusage_id):
        return self.call(context, self.make_msg('delete_pool_usage',
                                                poolusage_id=poolusage_id))


# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (c) 2012 OpenStack, LLC
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Request Body limiting middleware.

"""

from oslo.config import cfg
import webob.dec
import webob.exc

from vsm import flags
from vsm.openstack.common import log as logging
from vsm import wsgi

#default request size is 112k
max_request_body_size_opt = cfg.IntOpt('osapi_max_request_body_size',
                                       default=114688,
                                       help='Max size for body of a request')

FLAGS = flags.FLAGS
FLAGS.register_opt(max_request_body_size_opt)
LOG = logging.getLogger(__name__)

class LimitingReader(object):
    """Reader to limit the size of an incoming request."""
    def __init__(self, data, limit):
        """
        :param data: Underlying data object
        :param limit: maximum number of bytes the reader should allow
        """
        self.data = data
        self.limit = limit
        self.bytes_read = 0

    def __iter__(self):
        for chunk in self.data:
            self.bytes_read += len(chunk)
            if self.bytes_read > self.limit:
                msg = _("Request is too large.")
                raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg)
            else:
                yield chunk

    def read(self, i=None):
        result = self.data.read(i)
        self.bytes_read += len(result)
        if self.bytes_read > self.limit:
            msg = _("Request is too large.")
            raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg)
        return result

class RequestBodySizeLimiter(wsgi.Middleware):
    """Add a 'vsm.context' to WSGI environ."""

    def __init__(self, *args, **kwargs):
        super(RequestBodySizeLimiter, self).__init__(*args, **kwargs)

    @webob.dec.wsgify(RequestClass=wsgi.Request)
    def __call__(self, req):
        if req.content_length > FLAGS.osapi_max_request_body_size:
            msg = _("Request is too large.")
            raise webob.exc.HTTPRequestEntityTooLarge(explanation=msg)
        if req.content_length is None and req.is_body_readable:
            limiter = LimitingReader(req.body_file,
                                     FLAGS.osapi_max_request_body_size)
            req.body_file = limiter
        return self.application



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

import webob
from webob import exc
from vsm.api.openstack import wsgi
from vsm.api import xmlutil
from vsm import flags
from vsm.openstack.common import log as logging
from vsm.api.views import mdses as mds_views
from vsm.api.views import summary as summary_view
from vsm import conductor
from vsm import scheduler
from vsm import exception

from vsm import db
LOG = logging.getLogger(__name__)

FLAGS = flags.FLAGS

def make_mds(elem, detailed=False):
    elem.set('id')
    elem.set('mds_name')
    elem.set('gid')
    elem.set('state')
    elem.set('address')

    if detailed:
        pass

mds_nsmap = {None: xmlutil.XMLNS_V11, 'atom': xmlutil.XMLNS_ATOM}

class OSDTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('mds', selector='mds')
        make_mds(root, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=mds_nsmap)

class OSDsTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = xmlutil.TemplateElement('mdses')
        elem = xmlutil.SubTemplateElement(root, 'mds', selector='mdsess')
        make_mds(elem, detailed=True)
        return xmlutil.MasterTemplate(root, 1, nsmap=mds_nsmap)

class Controller(wsgi.Controller):
    """The OSDs API controller for the OpenStack API."""
    _view_builder_class = mds_views.ViewBuilder

    def __init__(self, ext_mgr):
        self.conductor_api = conductor.API()
        self.scheduler_api = scheduler.API()
        self.ext_mgr = ext_mgr
        super(Controller, self).__init__()

    #def _get_zone_search_options(self):
    #    """Return zone search options allowed by non-admin."""
    #    return ('id', 'name', 'public_ip')

    def _get_mds(self, context, req, id):
        """Utility function for looking up an instance by id."""
        try:
            mds = self.conductor_api.mds_get(context, id)
        except exception.NotFound:
            msg = _("MDS could not be found")
            raise exc.HTTPNotFound(explanation=msg)
        return mds

    @wsgi.serializers(xml=OSDsTemplate)
    def show(self, req, id):
        """Return data about the given mds."""
        context = req.environ['vsm.context']

        try:
            mds = self._get_mds(context, req, id)
            error = self.conductor_api.ceph_error(context)
            if error:
                mds['state'] = error
        except exception.NotFound:
            raise exc.HTTPNotFound()

        return {'mds': mds}

    @wsgi.serializers(xml=OSDsTemplate)
    def index(self, req):
        """Get mds list."""
        #search_opts = {}
        #search_opts.update(req.GET)
        context = req.environ['vsm.context']
        #remove_invalid_options(context,
        #                        search_opts,
        #                        self._get_zone_search_options)
        #zones = self.conductor_api.get_zone_list(context)
        error = self.conductor_api.ceph_error(context)
        mdses = self.conductor_api.mds_get_all(context)
        if error:
            for mds in mdses:
                mds['state'] = error

        LOG.info('vsm/api/v1/mds.py mdss:%s' % mdses)

        return self._view_builder.index(req, mdses)

    @wsgi.serializers(xml=OSDsTemplate)
    def detail(self, req):

        """Get mds list."""
        #search_opts = {}
        #search_opts.update(req.GET)
        context = req.environ['vsm.context']
        #remove_invalid_options(context,
        #                        search_opts,
        #                        self._get_zone_search_options)
        #zones = self.conductor_api.get_zone_list(context)
        mdses = self.conductor_api.mds_get_all(context)
        LOG.info('vsm/api/v1/mdses.py detailed mdses:%s' % mdses)

        return self._view_builder.detail(req, mdses)

    #TODO scheduler exe
    @wsgi.response(204)
    def delete(self, req, id):
        """delete a mds."""
        LOG.info("mds_delete")
        context = req.environ['vsm.context']
        mds = self._get_mds(context, req, id)
        self.conductor_api.mds_delete(context, mds['id'])

    @wsgi.response(202)
    @wsgi.serializers(xml=OSDsTemplate)
    @wsgi.action('remove')
    def _action_remove(self, req, id, body):
        context = req.environ['vsm.context']
        mds = self._get_mds(context, req, id)

        LOG.info("action_remove")
        mds = self._get_mds(context, req, id)
        #self.conductor_api.mds_remove(context, mds['id'])
        self.scheduler_api.mds_remove(context, mds)
        return webob.Response(status_int=202)

    @wsgi.response(202)
    @wsgi.serializers(xml=OSDsTemplate)
    @wsgi.action('restart')
    def _action_restart(self, req, id, body):
        context = req.environ['vsm.context']
        mds = self._get_mds(context, req, id)

        LOG.info("action_restart")
        self.scheduler_api.mds_restart(context, mds)
        return webob.Response(status_int=202)

    @wsgi.response(202)
    @wsgi.serializers(xml=OSDsTemplate)
    @wsgi.action('restore')
    def _action_restore(self, req, id, body):
        context = req.environ['vsm.context']
        mds = self._get_mds(context, req, id)

        LOG.info("action_restore")
        mds['operation_status'] = FLAGS.vsm_status_present
        self.conductor_api.\
             mds_state_update_or_create(context, \
             mds, create=False)
        self.scheduler_api.mds_restore(context, mds)
        return webob.Response(status_int=202)

    def summary(self, req, body=None,cluster_id=None):
        LOG.info('CEPH_LOG mds-summary body %s ' % body)
        context = req.environ['vsm.context']
        if cluster_id:
            mds = db.summary_get_by_cluster_id_and_type(context, cluster_id, 'mds')
        else:
            mds = db.summary_get_by_type_first(context, 'mds')
        vb = summary_view.ViewBuilder()
        return vb.basic(mds, 'mds')

def create_resource(ext_mgr):
    return wsgi.Resource(Controller(ext_mgr))

#def remove_invalid_options(context, search_options, allowed_search_options):
#    """Remove search options that are not valid for non-admin API/context."""
#    if context.is_admin:
#        # Allow all options
#        return
#    # Otherwise, strip out all unknown options
#    unknown_options = [opt for opt in search_options
#                       if opt not in allowed_search_options]
#    bad_options = ", ".join(unknown_options)
#    log_msg = _("Removing options '%(bad_options)s' from query") % locals()
#    LOG.debug(log_msg)
#    for opt in unknown_options:
#        del search_options[opt]



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright (c) 2010 OpenStack, LLC.
# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Keys Service
"""

from vsm import flags
from vsm.openstack.common import log as logging

LOG = logging.getLogger(__name__)
FLAGS = flags.FLAGS

class SSHManager(object):
    """Chooses a host to create storages."""

    def __init__(self, fpath=FLAGS.authorized_keys):
        self._fpath = fpath or FLAGS.authorized_keys
        self._key_list = open(self._fpath, 'r').readlines()

    def _get_hostname(self, key):
        if key.find('@') != -1:
            return key.split('@')[1].strip()

    def _flush_key_file(self):
        fd = open(self._fpath, 'w')
        for k in self._key_list:
            fd.write(k + '\n')
        fd.close()

    def _find_key(self, key):
        for k in self._key_list:
            if k.find(key.strip()) != -1:
                return True

        return False

    def _append_key(self, key):
        if self._find_key(key) == False:
            self._key_list.append(key)
            fd = open(self._fpath, 'a+')
            fd.write(key)
            fd.close()

    def _is_new_key(self, key):
        host = self._get_hostname(key)
        for k in self._key_list:
            h = self._get_hostname(k)
            if h.find(host) != -1 or host.find(h) != -1:
                return False
        return True

    def _find_old_key(self, key):
        host = self._get_hostname(key)
        for k in self._key_list:
            h = self._get_hostname(k)
            if h.find(host) != -1 or host.find(h) != -1:
                return k
        return None

    def _update_key(self, key):
        if self._find_key(key) == False:
            if self._is_new_key(key):
                self._key_list.remove(self._find_old_key(key))
                self._flush_key_file()
            else:
                self._append_key(key)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2012 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
A helper class for proxy objects to remote APIs.

For more information about rpc API version numbers, see:
    rpc/dispatcher.py
"""

import os
from vsm.openstack.common import rpc

class RpcProxy(object):
    """A helper class for rpc clients.

    This class is a wrapper around the RPC client API.  It allows you to
    specify the topic and API version in a single place.  This is intended to
    be used as a base class for a class that implements the client side of an
    rpc API.
    """

    def __init__(self, topic, default_version):
        """Initialize an RpcProxy.

        :param topic: The topic to use for all messages.
        :param default_version: The default API version to request in all
               outgoing messages.  This can be overridden on a per-message
               basis.
        """
        self.topic = topic
        self.default_version = default_version
        self.host = os.popen('hostname').read().strip()
        super(RpcProxy, self).__init__()

    def _set_version(self, msg, vers):
        """Helper method to set the version in a message.

        :param msg: The message having a version added to it.
        :param vers: The version number to add to the message.
        """
        msg['version'] = vers if vers else self.default_version

    def _get_topic(self, topic):
        """Return the topic to use for a message."""
        return topic if topic else self.topic

    @staticmethod
    def make_msg(method, **kwargs):
        return {'method': method, 'args': kwargs}

    def call(self, context, msg, topic=None, version=None, timeout=None, need_try=True):
        """rpc.call() a remote method.

        :param context: The request context
        :param msg: The message to send, including the method and args.
        :param topic: Override the topic for this message.
        :param timeout: (Optional) A timeout to use when waiting for the
               response.  If no timeout is specified, a default timeout will be
               used that is usually sufficient.
        :param version: (Optional) Override the requested API version in this
               message.

        :returns: The return value from the remote method.
        """
        self._set_version(msg, version)
        return rpc.call(context, self._get_topic(topic), msg, timeout, need_try=need_try)

    def multicall(self, context, msg, topic=None, version=None, timeout=None, need_try=True):
        """rpc.multicall() a remote method.

        :param context: The request context
        :param msg: The message to send, including the method and args.
        :param topic: Override the topic for this message.
        :param timeout: (Optional) A timeout to use when waiting for the
               response.  If no timeout is specified, a default timeout will be
               used that is usually sufficient.
        :param version: (Optional) Override the requested API version in this
               message.

        :returns: An iterator that lets you process each of the returned values
                  from the remote method as they arrive.
        """
        self._set_version(msg, version)
        return rpc.multicall(context, self._get_topic(topic), msg, timeout, need_try=need_try)

    def cast(self, context, msg, topic=None, version=None, need_try=True):
        """rpc.cast() a remote method.

        :param context: The request context
        :param msg: The message to send, including the method and args.
        :param topic: Override the topic for this message.
        :param version: (Optional) Override the requested API version in this
               message.

        :returns: None.  rpc.cast() does not wait on any return value from the
                  remote method.
        """
        self._set_version(msg, version)
        rpc.cast(context, self._get_topic(topic), msg, need_try)

    def fanout_cast(self, context, msg, topic=None, version=None, need_try=True):
        """rpc.fanout_cast() a remote method.

        :param context: The request context
        :param msg: The message to send, including the method and args.
        :param topic: Override the topic for this message.
        :param version: (Optional) Override the requested API version in this
               message.

        :returns: None.  rpc.fanout_cast() does not wait on any return value
                  from the remote method.
        """
        self._set_version(msg, version)
        rpc.fanout_cast(context, self._get_topic(topic), msg, need_try)

    def cast_to_server(self, context, server_params, msg, topic=None,
                       version=None, need_try=True):
        """rpc.cast_to_server() a remote method.

        :param context: The request context
        :param server_params: Server parameters.  See rpc.cast_to_server() for
               details.
        :param msg: The message to send, including the method and args.
        :param topic: Override the topic for this message.
        :param version: (Optional) Override the requested API version in this
               message.

        :returns: None.  rpc.cast_to_server() does not wait on any
                  return values.
        """
        self._set_version(msg, version)
        rpc.cast_to_server(context, server_params, self._get_topic(topic), msg, need_try)

    def fanout_cast_to_server(self, context, server_params, msg, topic=None,
                              version=None, need_try=True):
        """rpc.fanout_cast_to_server() a remote method.

        :param context: The request context
        :param server_params: Server parameters.  See rpc.cast_to_server() for
               details.
        :param msg: The message to send, including the method and args.
        :param topic: Override the topic for this message.
        :param version: (Optional) Override the requested API version in this
               message.

        :returns: None.  rpc.fanout_cast_to_server() does not wait on any
                  return values.
        """
        self._set_version(msg, version)
        rpc.fanout_cast_to_server(context, server_params,
                                  self._get_topic(topic), msg, need_try)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from vsm.api import common
import logging

import time
LOG = logging.getLogger(__name__)

class ViewBuilder(common.ViewBuilder):
    _collection_name = "rbd_pools"

    def basic(self, request, rbd_pool):
        LOG.info("rbd_pools api view %s " % rbd_pool)
        rbd = {
            "rbd_pool": {
                "id": rbd_pool['id'],
                "pool": rbd_pool['pool'],
                "image_name": rbd_pool['image'],
                "size": rbd_pool['size'],
                "objects": rbd_pool['objects'],
                "order": rbd_pool['order'],
                "format": rbd_pool['format'],
            }
        }
        rbd['rbd_pool']['updated_at'] = time.strftime("%Y-%m-%d %H:%M:%S", time.strptime(rbd_pool['updated_at'], "%Y-%m-%dT%H:%M:%S.000000"))
        return rbd

    def _detail(self, request, rbd_pool):
        LOG.info("rbd_pools api detail view %s " % rbd_pool)
        rbd = {
            "rbd_pool": {
                "id": rbd_pool['id'],
                "pool": rbd_pool['pool'],
                "image_name": rbd_pool['image'],
                "size": rbd_pool['size'],
                "objects": rbd_pool['objects'],
                "order": rbd_pool['order'],
                "format": rbd_pool['format'],
            }
        }
        try:
            rbd['rbd_pool']['updated_at'] = time.strftime("%Y-%m-%d %H:%M:%S", time.strptime(rbd_pool['updated_at'], "%Y-%m-%dT%H:%M:%S.000000"))
        except:
            rbd['rbd_pool']['updated_at'] = time.strftime("%Y-%m-%d %H:%M:%S", time.strptime(rbd_pool['created_at'], "%Y-%m-%dT%H:%M:%S.000000"))
        return rbd

    def detail(self, request, rbd_pools):
        return self._list_view(self._detail, request, rbd_pools)

    def index(self, request, rbd_pools):
        """Show a list of rbd_pools without many details."""
        return self._list_view(self.basic, request, rbd_pools)

    def _list_view(self, func, request, rbd_pools):
        """Provide a view for a list of rbd_pools."""
        rbd_pool_list = [func(request, rbd_pool)["rbd_pool"] for rbd_pool in rbd_pools]
        rbd_pools_dict = dict(rbd_pools=rbd_pool_list)
        return rbd_pools_dict




#    Copyright 2014 Intel
#    All Rights Reserved
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Used by Agent Service to connect with Controller.
In order to get key parameters to set configuration files.
"""

import time
from vsm.manifest import config
from vsm.manifest.wsgi_client import WSGIClient
from vsm.manifest.parser import ManifestParser
from vsm.openstack.common import log as logging
from vsm import flags

LOG = logging.getLogger(__name__)
FLAGS = flags.FLAGS

class AgentChecker(object):
    """Agent Checker is used to check agent services.

    AgentChecker is used to add into vsm clusters.
    if vsm_is_configed:
        start_as_usuall
    else:
        get_config_from_controller.
        config_vsm_files
    """
    def __init__(self, fpath=FLAGS.server_manifest):
        self._file_path = fpath
        self._smp = ManifestParser(self._file_path)
        self._send_data = self._smp.format_to_json(check_manifest_tag=True)
        self._server_host = self._send_data['vsm_controller_ip']
        self._sender = WSGIClient(self._server_host,
                        self._send_data['auth_key'])

    def config_vsm_files(self):
        """Begin to config vsm configuration files."""
        # If vsm.conf changes, vsm-agent update the configuration files.
        # need to add file comparation operation.
        can_connect = False
        while can_connect == False:
            try:
                recive_data = self._sender.index()
                can_connect = True
            except:
                time.sleep(10)
                LOG.info('Can not connect to vsm-api. reconnect.')

        # If find the keyring_admin update it?
        if recive_data.get('keyring_admin', None):
            LOG.info('Get keyring.admin from DB.')
            config.set_keyring_admin(recive_data['keyring_admin'])
        else:
            LOG.info('Can not get keyring from DB.')

        if not config.is_vsm_ok():
            config.set_vsm_conf(recive_data)
            return True
        else:
            LOG.info('vsm.conf is ok now. skip this configuration.')
            return False



# Copyright (c) 2011-2012 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from vsm.openstack.common.scheduler import filters

class AvailabilityZoneFilter(filters.BaseHostFilter):
    """Filters Hosts by availability zone."""

    def host_passes(self, host_state, filter_properties):
        spec = filter_properties.get('request_spec', {})
        props = spec.get('resource_properties', [])
        availability_zone = props.get('availability_zone')

        if availability_zone:
            return availability_zone == host_state.service['availability_zone']
        return True



# Copyright (c) 2011 OpenStack Foundation.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from vsm.openstack.common import log as logging
from vsm.openstack.common.scheduler import filters
from vsm.openstack.common.scheduler.filters import extra_specs_ops

LOG = logging.getLogger(__name__)

class CapabilitiesFilter(filters.BaseHostFilter):
    """HostFilter to work with resource (instance & storage) type records."""

    def _satisfies_extra_specs(self, capabilities, resource_type):
        """Check that the capabilities provided by the services
        satisfy the extra specs associated with the instance type"""
        extra_specs = resource_type.get('extra_specs', [])
        if not extra_specs:
            return True

        for key, req in extra_specs.iteritems():
            # Either not scope format, or in capabilities scope
            scope = key.split(':')
            if len(scope) > 1 and scope[0] != "capabilities":
                continue
            elif scope[0] == "capabilities":
                del scope[0]

            cap = capabilities
            for index in range(0, len(scope)):
                try:
                    cap = cap.get(scope[index], None)
                except AttributeError:
                    return False
                if cap is None:
                    return False
            if not extra_specs_ops.match(cap, req):
                return False
        return True

    def host_passes(self, host_state, filter_properties):
        """Return a list of hosts that can create instance_type."""
        # Note(zhiteng) Currently only Vsm and Nova are using
        # this filter, so the resource type is either instance or
        # storage.
        resource_type = filter_properties.get('resource_type')
        if not self._satisfies_extra_specs(host_state.capabilities,
                                           resource_type):
            return False
        return True



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Defines interface for DB access.

The underlying driver is loaded as a :class:`LazyPluggable`.

Functions in this module are imported into the vsm.db namespace. Call these
functions from vsm.db namespace, not the vsm.db.api namespace.

All functions in this module return objects that implement a dictionary-like
interface. Currently, many of these objects are sqlalchemy objects that
implement a dictionary interface. However, a future goal is to have all of
these objects be simple dictionaries.

**Related Flags**

:db_backend:  string to lookup in the list of LazyPluggable backends.
              `sqlalchemy` is the only supported backend right now.

:sql_connection:  string specifying the sqlalchemy connection to use, like:
                  `sqlite:///var/lib/vsm/vsm.sqlite`.

:enable_new_services:  when adding a new service to the database, is it in the
                       pool of available storage (Default: True)

"""

from oslo.config import cfg

from vsm import exception
from vsm import flags
from vsm import utils

db_opts = [
    cfg.StrOpt('db_backend',
               default='sqlalchemy',
               help='The backend to use for db'),
    cfg.BoolOpt('enable_new_services',
                default=True,
                help='Services to be added to the available pool on create'),
    cfg.StrOpt('storage_name_template',
               default='storage-%s',
               help='Template string to be used to generate storage names'),
    cfg.StrOpt('snapshot_name_template',
               default='snapshot-%s',
               help='Template string to be used to generate snapshot names'),
    cfg.StrOpt('backup_name_template',
               default='backup-%s',
               help='Template string to be used to generate backup names'), ]

FLAGS = flags.FLAGS
FLAGS.register_opts(db_opts)

IMPL = utils.LazyPluggable('db_backend',
                           sqlalchemy='vsm.db.sqlalchemy.api')

class NoMoreTargets(exception.VsmException):
    """No more available targets"""
    pass

###################

def device_get_by_name(context, device_name):
    return IMPL.device_get_by_name(context, device_name)

def service_destroy(context, service_id):
    """Destroy the service or raise if it does not exist."""
    return IMPL.service_destroy(context, service_id)

def service_get(context, service_id):
    """Get a service or raise if it does not exist."""
    return IMPL.service_get(context, service_id)

def service_get_by_host_and_topic(context, host, topic):
    """Get a service by host it's on and topic it listens to."""
    return IMPL.service_get_by_host_and_topic(context, host, topic)

def service_get_all(context, disabled=None):
    """Get all services."""
    return IMPL.service_get_all(context, disabled)

def service_get_all_by_topic(context, topic):
    """Get all services for a given topic."""
    return IMPL.service_get_all_by_topic(context, topic)

def service_get_all_by_host(context, host):
    """Get all services for a given host."""
    return IMPL.service_get_all_by_host(context, host)

def service_get_all_bmc_by_host(context, host):
    """Get all compute services for a given host."""
    return IMPL.service_get_all_bmc_by_host(context, host)

def service_get_all_storage_sorted(context):
    """Get all storage services sorted by storage count.

    :returns: a list of (Service, storage_count) tuples.

    """
    return IMPL.service_get_all_storage_sorted(context)

def service_get_by_args(context, host, binary):
    """Get the state of an service by node name and binary."""
    return IMPL.service_get_by_args(context, host, binary)

def service_create(context, values):
    """Create a service from the values dictionary."""
    return IMPL.service_create(context, values)

def service_update(context, service_id, values):
    """Set the given properties on an service and update it.

    Raises NotFound if service does not exist.

    """
    return IMPL.service_update(context, service_id, values)

def migration_update(context, id, values):
    """Update a migration instance."""
    return IMPL.migration_update(context, id, values)

def migration_create(context, values):
    """Create a migration record."""
    return IMPL.migration_create(context, values)

def migration_get(context, migration_id):
    """Finds a migration by the id."""
    return IMPL.migration_get(context, migration_id)

def migration_get_by_instance_and_status(context, instance_uuid, status):
    """Finds a migration by the instance uuid its migrating."""
    return IMPL.migration_get_by_instance_and_status(context,
                                                     instance_uuid,
                                                     status)

def migration_get_all_unconfirmed(context, confirm_window):
    """Finds all unconfirmed migrations within the confirmation window."""
    return IMPL.migration_get_all_unconfirmed(context, confirm_window)

###################

def iscsi_target_count_by_host(context, host):
    """Return count of export devices."""
    return IMPL.iscsi_target_count_by_host(context, host)

def iscsi_target_create_safe(context, values):
    """Create an iscsi_target from the values dictionary.

    The device is not returned. If the create violates the unique
    constraints because the iscsi_target and host already exist,
    no exception is raised.

    """
    return IMPL.iscsi_target_create_safe(context, values)

###############

def storage_allocate_iscsi_target(context, storage_id, host):
    """Atomically allocate a free iscsi_target from the pool."""
    return IMPL.storage_allocate_iscsi_target(context, storage_id, host)

def storage_attached(context, storage_id, instance_id, mountpoint):
    """Ensure that a storage is set as attached."""
    return IMPL.storage_attached(context, storage_id, instance_id, mountpoint)

def storage_create(context, values):
    """Create a storage from the values dictionary."""
    return IMPL.storage_create(context, values)

def storage_data_get_for_host(context, host, session=None):
    """Get (storage_count, gigabytes) for project."""
    return IMPL.storage_data_get_for_host(context,
                                         host,
                                         session)

def storage_data_get_for_project(context, project_id, session=None):
    """Get (storage_count, gigabytes) for project."""
    return IMPL.storage_data_get_for_project(context,
                                            project_id,
                                            session)

def storage_destroy(context, storage_id):
    """Destroy the storage or raise if it does not exist."""
    return IMPL.storage_destroy(context, storage_id)

def storage_detached(context, storage_id):
    """Ensure that a storage is set as detached."""
    return IMPL.storage_detached(context, storage_id)

def storage_get(context, storage_id):
    """Get a storage or raise if it does not exist."""
    return IMPL.storage_get(context, storage_id)

def storage_get_all(context, marker, limit, sort_key, sort_dir):
    """Get all storages."""
    return IMPL.storage_get_all(context, marker, limit, sort_key, sort_dir)

def storage_get_all_by_host(context, host):
    """Get all storages belonging to a host."""
    return IMPL.storage_get_all_by_host(context, host)

def storage_get_all_by_instance_uuid(context, instance_uuid):
    """Get all storages belonging to a instance."""
    return IMPL.storage_get_all_by_instance_uuid(context, instance_uuid)

def storage_get_all_by_project(context, project_id, marker, limit, sort_key,
                              sort_dir):
    """Get all storages belonging to a project."""
    return IMPL.storage_get_all_by_project(context, project_id, marker, limit,
                                          sort_key, sort_dir)

def storage_get_iscsi_target_num(context, storage_id):
    """Get the target num (tid) allocated to the storage."""
    return IMPL.storage_get_iscsi_target_num(context, storage_id)

def storage_update(context, storage_id, values):
    """Set the given properties on an storage and update it.

    Raises NotFound if storage does not exist.

    """
    return IMPL.storage_update(context, storage_id, values)

####################

def snapshot_create(context, values):
    """Create a snapshot from the values dictionary."""
    return IMPL.snapshot_create(context, values)

def snapshot_destroy(context, snapshot_id):
    """Destroy the snapshot or raise if it does not exist."""
    return IMPL.snapshot_destroy(context, snapshot_id)

def snapshot_get(context, snapshot_id):
    """Get a snapshot or raise if it does not exist."""
    return IMPL.snapshot_get(context, snapshot_id)

def snapshot_get_all(context):
    """Get all snapshots."""
    return IMPL.snapshot_get_all(context)

def snapshot_get_all_by_project(context, project_id):
    """Get all snapshots belonging to a project."""
    return IMPL.snapshot_get_all_by_project(context, project_id)

def snapshot_get_all_for_storage(context, storage_id):
    """Get all snapshots for a storage."""
    return IMPL.snapshot_get_all_for_storage(context, storage_id)

def snapshot_update(context, snapshot_id, values):
    """Set the given properties on an snapshot and update it.

    Raises NotFound if snapshot does not exist.

    """
    return IMPL.snapshot_update(context, snapshot_id, values)

def snapshot_data_get_for_project(context, project_id, session=None):
    """Get count and gigabytes used for snapshots for specified project."""
    return IMPL.snapshot_data_get_for_project(context,
                                              project_id,
                                              session)

####################

def snapshot_metadata_get(context, snapshot_id):
    """Get all metadata for a snapshot."""
    return IMPL.snapshot_metadata_get(context, snapshot_id)

def snapshot_metadata_delete(context, snapshot_id, key):
    """Delete the given metadata item."""
    IMPL.snapshot_metadata_delete(context, snapshot_id, key)

def snapshot_metadata_update(context, snapshot_id, metadata, delete):
    """Update metadata if it exists, otherwise create it."""
    IMPL.snapshot_metadata_update(context, snapshot_id, metadata, delete)

####################

def storage_metadata_get(context, storage_id):
    """Get all metadata for a storage."""
    return IMPL.storage_metadata_get(context, storage_id)

def storage_metadata_delete(context, storage_id, key):
    """Delete the given metadata item."""
    IMPL.storage_metadata_delete(context, storage_id, key)

def storage_metadata_update(context, storage_id, metadata, delete):
    """Update metadata if it exists, otherwise create it."""
    IMPL.storage_metadata_update(context, storage_id, metadata, delete)

##################

def storage_type_create(context, values):
    """Create a new storage type."""
    return IMPL.storage_type_create(context, values)

def storage_type_get_all(context, inactive=False):
    """Get all storage types."""
    return IMPL.storage_type_get_all(context, inactive)

def storage_type_get(context, id):
    """Get storage type by id."""
    return IMPL.storage_type_get(context, id)

def storage_type_get_by_name(context, name):
    """Get storage type by name."""
    return IMPL.storage_type_get_by_name(context, name)

def storage_type_destroy(context, id):
    """Delete a storage type."""
    return IMPL.storage_type_destroy(context, id)

def storage_get_active_by_window(context, begin, end=None, project_id=None):
    """Get all the storages inside the window.

    Specifying a project_id will filter for a certain project."""
    return IMPL.storage_get_active_by_window(context, begin, end, project_id)

####################

def storage_type_extra_specs_get(context, storage_type_id):
    """Get all extra specs for a storage type."""
    return IMPL.storage_type_extra_specs_get(context, storage_type_id)

def storage_type_extra_specs_delete(context, storage_type_id, key):
    """Delete the given extra specs item."""
    IMPL.storage_type_extra_specs_delete(context, storage_type_id, key)

def storage_type_extra_specs_update_or_create(context,
                                             storage_type_id,
                                             extra_specs):
    """Create or update storage type extra specs. This adds or modifies the
    key/value pairs specified in the extra specs dict argument"""
    IMPL.storage_type_extra_specs_update_or_create(context,
                                                  storage_type_id,
                                                  extra_specs)

###################

def storage_glance_metadata_create(context, storage_id, key, value):
    """Update the Glance metadata for the specified storage."""
    return IMPL.storage_glance_metadata_create(context,
                                              storage_id,
                                              key,
                                              value)

def storage_glance_metadata_get(context, storage_id):
    """Return the glance metadata for a storage."""
    return IMPL.storage_glance_metadata_get(context, storage_id)

def storage_snapshot_glance_metadata_get(context, snapshot_id):
    """Return the Glance metadata for the specified snapshot."""
    return IMPL.storage_snapshot_glance_metadata_get(context, snapshot_id)

def storage_glance_metadata_copy_to_snapshot(context, snapshot_id, storage_id):
    """
    Update the Glance metadata for a snapshot by copying all of the key:value
    pairs from the originating storage. This is so that a storage created from
    the snapshot will retain the original metadata.
    """
    return IMPL.storage_glance_metadata_copy_to_snapshot(context, snapshot_id,
                                                        storage_id)

def storage_glance_metadata_copy_to_storage(context, storage_id, snapshot_id):
    """
    Update the Glance metadata from a storage (created from a snapshot) by
    copying all of the key:value pairs from the originating snapshot. This is
    so that the Glance metadata from the original storage is retained.
    """
    return IMPL.storage_glance_metadata_copy_to_storage(context, storage_id,
                                                      snapshot_id)

def storage_glance_metadata_delete_by_storage(context, storage_id):
    """Delete the glance metadata for a storage."""
    return IMPL.storage_glance_metadata_delete_by_storage(context, storage_id)

def storage_glance_metadata_delete_by_snapshot(context, snapshot_id):
    """Delete the glance metadata for a snapshot."""
    return IMPL.storage_glance_metadata_delete_by_snapshot(context, snapshot_id)

def storage_glance_metadata_copy_from_storage_to_storage(context,
                                                      src_storage_id,
                                                      storage_id):
    """
    Update the Glance metadata for a storage by copying all of the key:value
    pairs from the originating storage. This is so that a storage created from
    the storage (clone) will retain the original metadata.
    """
    return IMPL.storage_glance_metadata_copy_from_storage_to_storage(
        context,
        src_storage_id,
        storage_id)

###################

def sm_backend_conf_create(context, values):
    """Create a new SM Backend Config entry."""
    return IMPL.sm_backend_conf_create(context, values)

def sm_backend_conf_update(context, sm_backend_conf_id, values):
    """Update a SM Backend Config entry."""
    return IMPL.sm_backend_conf_update(context, sm_backend_conf_id, values)

def sm_backend_conf_delete(context, sm_backend_conf_id):
    """Delete a SM Backend Config."""
    return IMPL.sm_backend_conf_delete(context, sm_backend_conf_id)

def sm_backend_conf_get(context, sm_backend_conf_id):
    """Get a specific SM Backend Config."""
    return IMPL.sm_backend_conf_get(context, sm_backend_conf_id)

def sm_backend_conf_get_by_sr(context, sr_uuid):
    """Get a specific SM Backend Config."""
    return IMPL.sm_backend_conf_get_by_sr(context, sr_uuid)

def sm_backend_conf_get_all(context):
    """Get all SM Backend Configs."""
    return IMPL.sm_backend_conf_get_all(context)

####################

def sm_flavor_create(context, values):
    """Create a new SM Flavor entry."""
    return IMPL.sm_flavor_create(context, values)

def sm_flavor_update(context, sm_flavor_id, values):
    """Update a SM Flavor entry."""
    return IMPL.sm_flavor_update(context, values)

def sm_flavor_delete(context, sm_flavor_id):
    """Delete a SM Flavor."""
    return IMPL.sm_flavor_delete(context, sm_flavor_id)

def sm_flavor_get(context, sm_flavor):
    """Get a specific SM Flavor."""
    return IMPL.sm_flavor_get(context, sm_flavor)

def sm_flavor_get_all(context):
    """Get all SM Flavors."""
    return IMPL.sm_flavor_get_all(context)

####################

def sm_storage_create(context, values):
    """Create a new child Zone entry."""
    return IMPL.sm_storage_create(context, values)

def sm_storage_update(context, storage_id, values):
    """Update a child Zone entry."""
    return IMPL.sm_storage_update(context, values)

def sm_storage_delete(context, storage_id):
    """Delete a child Zone."""
    return IMPL.sm_storage_delete(context, storage_id)

def sm_storage_get(context, storage_id):
    """Get a specific child Zone."""
    return IMPL.sm_storage_get(context, storage_id)

def sm_storage_get_all(context):
    """Get all child Zones."""
    return IMPL.sm_storage_get_all(context)

###################

def quota_create(context, project_id, resource, limit):
    """Create a quota for the given project and resource."""
    return IMPL.quota_create(context, project_id, resource, limit)

def quota_get(context, project_id, resource):
    """Retrieve a quota or raise if it does not exist."""
    return IMPL.quota_get(context, project_id, resource)

def quota_get_all_by_project(context, project_id):
    """Retrieve all quotas associated with a given project."""
    return IMPL.quota_get_all_by_project(context, project_id)

def quota_update(context, project_id, resource, limit):
    """Update a quota or raise if it does not exist."""
    return IMPL.quota_update(context, project_id, resource, limit)

def quota_destroy(context, project_id, resource):
    """Destroy the quota or raise if it does not exist."""
    return IMPL.quota_destroy(context, project_id, resource)

###################

def quota_class_create(context, class_name, resource, limit):
    """Create a quota class for the given name and resource."""
    return IMPL.quota_class_create(context, class_name, resource, limit)

def quota_class_get(context, class_name, resource):
    """Retrieve a quota class or raise if it does not exist."""
    return IMPL.quota_class_get(context, class_name, resource)

def quota_class_get_all_by_name(context, class_name):
    """Retrieve all quotas associated with a given quota class."""
    return IMPL.quota_class_get_all_by_name(context, class_name)

def quota_class_update(context, class_name, resource, limit):
    """Update a quota class or raise if it does not exist."""
    return IMPL.quota_class_update(context, class_name, resource, limit)

def quota_class_destroy(context, class_name, resource):
    """Destroy the quota class or raise if it does not exist."""
    return IMPL.quota_class_destroy(context, class_name, resource)

def quota_class_destroy_all_by_name(context, class_name):
    """Destroy all quotas associated with a given quota class."""
    return IMPL.quota_class_destroy_all_by_name(context, class_name)

###################

def quota_usage_create(context, project_id, resource, in_use, reserved,
                       until_refresh):
    """Create a quota usage for the given project and resource."""
    return IMPL.quota_usage_create(context, project_id, resource,
                                   in_use, reserved, until_refresh)

def quota_usage_get(context, project_id, resource):
    """Retrieve a quota usage or raise if it does not exist."""
    return IMPL.quota_usage_get(context, project_id, resource)

def quota_usage_get_all_by_project(context, project_id):
    """Retrieve all usage associated with a given resource."""
    return IMPL.quota_usage_get_all_by_project(context, project_id)

###################

def reservation_create(context, uuid, usage, project_id, resource, delta,
                       expire):
    """Create a reservation for the given project and resource."""
    return IMPL.reservation_create(context, uuid, usage, project_id,
                                   resource, delta, expire)

def reservation_get(context, uuid):
    """Retrieve a reservation or raise if it does not exist."""
    return IMPL.reservation_get(context, uuid)

def reservation_get_all_by_project(context, project_id):
    """Retrieve all reservations associated with a given project."""
    return IMPL.reservation_get_all_by_project(context, project_id)

def reservation_destroy(context, uuid):
    """Destroy the reservation or raise if it does not exist."""
    return IMPL.reservation_destroy(context, uuid)

###################

def quota_reserve(context, resources, quotas, deltas, expire,
                  until_refresh, max_age, project_id=None):
    """Check quotas and create appropriate reservations."""
    return IMPL.quota_reserve(context, resources, quotas, deltas, expire,
                              until_refresh, max_age, project_id=project_id)

def reservation_commit(context, reservations, project_id=None):
    """Commit quota reservations."""
    return IMPL.reservation_commit(context, reservations,
                                   project_id=project_id)

def reservation_rollback(context, reservations, project_id=None):
    """Roll back quota reservations."""
    return IMPL.reservation_rollback(context, reservations,
                                     project_id=project_id)

def quota_destroy_all_by_project(context, project_id):
    """Destroy all quotas associated with a given project."""
    return IMPL.quota_destroy_all_by_project(context, project_id)

def reservation_expire(context):
    """Roll back any expired reservations."""
    return IMPL.reservation_expire(context)

###################

def backup_get(context, backup_id):
    """Get a backup or raise if it does not exist."""
    return IMPL.backup_get(context, backup_id)

def backup_get_all(context):
    """Get all backups."""
    return IMPL.backup_get_all(context)

def backup_get_all_by_host(context, host):
    """Get all backups belonging to a host."""
    return IMPL.backup_get_all_by_host(context, host)

def backup_create(context, values):
    """Create a backup from the values dictionary."""
    return IMPL.backup_create(context, values)

def backup_get_all_by_project(context, project_id):
    """Get all backups belonging to a project."""
    return IMPL.backup_get_all_by_project(context, project_id)

def backup_update(context, backup_id, values):
    """
    Set the given properties on a backup and update it.

    Raises NotFound if backup does not exist.
    """
    return IMPL.backup_update(context, backup_id, values)

def backup_destroy(context, backup_id):
    """Destroy the backup or raise if it does not exist."""
    return IMPL.backup_destroy(context, backup_id)

#bellow is about osd state table operating
def osd_get(context, osd_id):
    """"get a entry according to specify osd_id"""
    return IMPL.osd_get(context, osd_id)

def osd_get_all(context, session=None):
    """get information about all osds"""
    return IMPL.osd_get_all(context, session)

def osd_get_all_down(context, session=None):
    """get information about all down osds"""
    return IMPL.osd_get_all_down(context, session)

def osd_get_all_up(context, session=None):
    """get information about all up osds"""
    return IMPL.osd_get_all_up(context, session)

def osd_get_by_service_id(context, service_id, session=None):
    """get osds by service id"""
    return IMPL.osd_get_by_service_id(context, service_id, session)

def osd_get_by_cluster_id(context, cluster_id, session=None):
    """get osds by cluster id"""
    return IMPL.osd_get_by_cluster_id(context, cluster_id, session)

def osd_destroy(context, osd_id):
    """Destory a osd."""
    return IMPL.osd_destroy(context, osd_id)

def osd_create(context, values):
    """create a osd from the values dictionary"""
    return IMPL.osd_create(context, values)

def osd_update(context, osd_id, values):
    """update a osd from the values dictionary"""
    return IMPL.osd_update(context, osd_id, values)

def osd_delete(context, osd_id):
    """delete a osd according to specify osd id"""
    return IMPL.osd_delete(context, osd_id)

#bellow is about the operation on crushmap
#@DEBUG
def crushmap_get(context, crushmap_id):
    """get a crushmap according to specify osd_id"""
    return IMPL.crushmap_get(context, crushmap_id)

def crushmap_get_all(context):
    """get all crushmaps"""
    return IMPL.crushmap_get_all(context)

def crushmap_create(context, values):
    """create a crushmap from the values dictionary"""
    return IMPL.crushmap_create(context, values)

def crushmap_update(context, crushmap_id, values):
    """update a crushmap from the values dictionary"""
    return IMPL.crushmap_update(context, crushmap_id, values)

def crushmap_delete(context, crushmap_id):
    """delete a osd according to specify crushmap id"""
    return IMPL.crushmap_delete(context, crushmap_id)

#bellow is about the operation on devices
def device_get(context, device_id):
    """get a device according to specify device_id"""
    return IMPL.device_get(context, device_id)

def device_get_all(context):
    """get all devices"""
    return IMPL.device_get_all(context)

def device_get_all_by_interface_type(context, interface_type):
    """get all devices by interface type"""
    return IMPL.device_get_all_by_interface_type(context, interface_type)

def device_get_all_by_device_type(context, device_type):
    """get all devices by device type"""
    return IMPL.device_get_all_by_device_type(context, device_type)

def device_get_all_by_service_id(context, service_id):
    """get all devices by service id"""
    return IMPL.device_get_all_by_service_id(context, service_id)

def device_get_all_by_state(context, state):
    """get all devices by state"""
    return IMPL.device_get_all_by_state(context, state)

def device_create(context, values):
    """create a device from the values dictionary"""
    return IMPL.device_create(context, values)

def device_update(context, device_id, values):
    """update a device from the values dictionary"""
    return IMPL.device_update(context, device_id, values)

def device_delete(context, device_id):
    """delete a device according to specify device id"""
    return IMPL.device_delete(context, device_id)

#bellow is about the operation on VSM user capacity management
def vsm_capacity_manage_get(context, vsm_capacity_manage_id):
    """get a user according to specify vsm_capacity_manage_id"""
    return IMPL.vsm_capacity_manage_get(context, vsm_capacity_manage_id)

def vsm_capacity_manage_get_all(context):
    """get all VSM users"""
    return IMPL.vsm_capacity_manage_get_all(context)

def vsm_capacity_manage_create(context, values):
    """create a VSM user from the values dictionary"""
    return IMPL.vsm_capacity_manage_create(context, values)

def vsm_capacity_manage_update(context, vsm_capacity_manage_id, values):
    """update a VSM user from the values dictionary"""
    return IMPL.vsm_capacity_manage_update(context, vsm_capacity_manage_id, values)

def vsm_capacity_manage_delete(context, vsm_capacity_manage_id):
    """delete a VSM user according to specify vsm_capacity_manage_id"""
    return IMPL.vsm_capacity_manage_delete(context, vsm_capacity_manage_id)

#bellow is about the operation on recipes
def recipe_get(context, recipe_id):
    """Get an recipe or raise if it does not exist."""
    return IMPL.recipe_get(context, recipe_id)

def recipe_get_all(context):
    """Get all recipes."""
    return IMPL.recipe_get_all(context)

def recipe_create(context, values):
    """Create a recipe from the values dictionary."""
    return IMPL.recipe_create(context, values)

def recipe_update(context, recipe_id, values):
    """Set the given properties on an recipe and update it.

    Raises NotFound if recipe does not exist.
    """
    return IMPL.recipe_update(context, recipe_id, values)

def recipe_delete(context, recipe_id):
    """Delete an recipe by recipe_id"""
    return IMPL.recipe_delete(context, recipe_id)

##storage pool
def pool_get_all(context):
    """get all pools"""
    return IMPL.pool_get_all(context)

def pool_get(context, pool_id):
    return IMPL.pool_get(context, pool_id)

def pool_get_by_db_id(context, id):
    return IMPL.pool_get_by_db_id(context, id)

def pool_get_by_name(context, pool_name, cluster_id):
    return IMPL.pool_get_by_name(context, pool_name, cluster_id)

def pool_create(context, values):
    """create a pool from the values dictionary"""
    return IMPL.pool_create(context, values)

def pool_update(context, pool_id, values):
    """update a pool with pool_id and values"""
    return IMPL.pool_update(context, pool_id, values)

def pool_update_by_name(context, pool_name, cluster_id, values):
    """update a pool with pool_name, cluster_id and values"""
    return IMPL.pool_update_by_name(context, pool_name, cluster_id, values)

#this pool_id is not the db id
def pool_update_by_pool_id(context, pool_id, values):
    return IMPL.pool_update_by_pool_id(context, pool_id, values)

def pool_get_by_ruleset(context, ruleset):
    return IMPL.pool_get_by_ruleset(context, ruleset)

def pool_destroy(context, pool_name):
    return IMPL.pool_destroy(context, pool_name)

#manange cluster
def cluster_get_all(context):
    """get all clusters"""
    return IMPL.cluster_get_all(context)

def cluster_create(context, values):
    """create a cluster from the values dictionary"""
    return IMPL.cluster_create(context, values)

def cluster_update(context, cluster_id, values):
    """update a cluster with cluster_id and values"""
    return IMPL.cluster_update(context, cluster_id, values)

def cluster_get(context, cluster_id):
    """Get an cluster with cluster_id"""
    return IMPL.cluster_get(context, cluster_id)

def cluster_increase_deleted_times(context, cluster_id):
    return IMPL.cluster_increase_deleted_times(context, cluster_id)

def cluster_update_ceph_conf(context, cluster_id, ceph_conf):
    return IMPL.cluster_update_ceph_conf(context, cluster_id, ceph_conf)

def cluster_get_ceph_conf(context, cluster_id):
    return IMPL.cluster_get_ceph_conf(context, cluster_id)

def cluster_get_deleted_times(context, cluster_id):
    return IMPL.cluster_get_deleted_times(context, cluster_id)

def cluster_info_dict_get_by_id(context, cluster_id):
    return IMPL.cluster_info_dict_get_by_id(context, cluster_id)

#manange storage_group
def storage_group_get_all(context):
    """get all storage_groups"""
    return IMPL.storage_group_get_all(context)

def storage_group_get_by_rule_id(context, rule_id):
    return IMPL.storage_group_get_by_rule_id(context, rule_id)

def storage_group_get_by_storage_class(context, storage_class):
    return IMPL.storage_group_get_by_storage_class(context, storage_class)

def storage_group_update_by_name(context, name, values):
    return IMPL.storage_group_update_by_name(context, name, values)

def update_storage_group_marker(context,name,marker):
    return IMPL.update_storage_group_marker(context,name,marker)

def create_storage_group(context, values):
    if values is None:
        return False

    res = storage_group_get_all(context)
    name_list = []
    for item in res:
        name_list.append(item['name'])

    if values['name'] not in name_list:
        storage_group_create(context, values)
    else:
        return False

    return True

def storage_group_create(context, values):
    """create a storage_group from the values dictionary"""
    return IMPL.storage_group_create(context, values)

def storage_group_update(context, group_id, values):
    """update a storage_group with group_id and values"""
    return IMPL.storage_group_update(context, group_id, values)

def storage_group_get(context, group_id):
    """Get an storage_group with group_id"""
    return IMPL.storage_group_get(context, group_id)

def storage_group_delete_by_order_and_name(context, take_order, name):
    return IMPL.storage_group_delete_by_order_and_name(context, take_order, name)

def storage_group_get_by_name(context, group_name):
    """Get an storage_group with group_id"""
    return IMPL.storage_group_get_by_name(context, group_name)
def get_size_by_storage_group_name(context,group_name):
    return IMPL.get_sum_choose_num_by_storage_group_name(context, group_name)



#manange zone
def zone_get_all(context):
    """get all zones"""
    return IMPL.zone_get_all(context)

def osd_locations_choices_by_type(context, type):
    return IMPL.osd_locations_choices_by_type(context,type)

def create_zone(context, values=None):
    if values is None:
        return False

    res = zone_get_all(context)
    zone_list = []
    for item in res:
        zone_list.append(item['name'])

    if values['name'] not in zone_list:
        zone_create(context, values)
    else:
        return True

    return True

def zone_create(context, values):
    """create a zone from the values dictionary"""
    return IMPL.zone_create(context, values)

def zone_update(context, zone_id, values):
    """update a zone with zone_id and values"""
    return IMPL.zone_update(context, zone_id, values)

def zone_get(context, zone_id):
    """Get an zone with zone_id"""
    return IMPL.zone_get(context, zone_id)

def service_get_by_host_and_topic_no_context(host, topic):
    """Get a service or raise if it does not exist."""
    return IMPL.service_get_by_host_and_topic_no_context(host, topic)

#server
def server_get_all(context):
    """"""
    return IMPL.server_get_all(context)
                                                      

def init_node_get_all(context):
    return IMPL.init_node_get_all(context)

def init_node_get(context, id):
    return IMPL.init_node_get(context, id)

def init_node_get_by_host(context, host, session=None):
    """Get init node ref by host name."""
    return IMPL.init_node_get_by_host(context, host, session)

def init_node_get_by_cluster_id(context, cluster_id, session=None):
    """Get init node ref by cluster id."""
    return IMPL.init_node_get_by_cluster_id(context, cluster_id, session)

def init_node_get_cluster_nodes(context, init_node_id, session=None):
    """Get active nodes in the same cluster."""
    return IMPL.init_node_get_cluster_nodes(context, init_node_id, session)
def init_node_get_by_device_id(context, device_id, session=None):
    """Get active nodes in the same cluster."""
    return IMPL.init_node_get_by_device_id(context, device_id, session)
#cluster
def cluster_get_by_name(context, values):
    return IMPL.cluster_get_by_name(context, values)

#init_node
def init_node_create(context, values):
    return IMPL.init_node_create(context, values)

def init_node_get_by_primary_public_ip(context, primary_public_ip):
    return IMPL.init_node_get_by_primary_public_ip(context, primary_public_ip)

def init_node_get_by_secondary_public_ip(context, secondary_public_ip):
    return IMPL.init_node_get_by_secondary_public_ip(context, \
                                                     secondary_public_ip)

def init_node_get_by_cluster_ip(context, cluster_ip):
    return IMPL.init_node_get_by_cluster_ip(context, cluster_ip)

def init_node_update(context, init_node_id, values):
    return IMPL.init_node_update(context, init_node_id, values)

def init_node_update_status_by_id(context, init_node_id, status):
    return IMPL.init_node_update_status_by_id(context, init_node_id, status)

def init_node_get_by_id_and_type(context, id, type):
    return IMPL.init_node_get_by_id_and_type(context, id, type)

def init_node_get_by_id(context, id):
    return IMPL.init_node_get_by_id(context, id)

def init_node_get_by_hostname(context, hostname, session=None):
    return IMPL.init_node_get_by_hostname(context, hostname, session)

def init_node_get_by_service_id(context, service_id, session=None):
    return IMPL.init_node_get_by_service_id(context, service_id, session)

def init_node_count_by_status(context, status):
    return IMPL.init_node_count_by_status(context, status)

#osd_state
def osd_state_create(context, values,force_create=False):
    return IMPL.osd_state_create(context, values,force_create=force_create)

def osd_state_get(context, osd_id):
    return IMPL.osd_state_get(context, osd_id)

def osd_state_get_all(context, limit=None, marker=None, sort_keys=None, sort_dir=None, search_opts={}):
    if search_opts:
        return IMPL.osd_state_get_sort_filter(context, limit, marker, sort_keys, sort_dir,search_opts)
    else:
        return IMPL.osd_state_get_all(context, limit, marker, sort_keys, sort_dir)

def osd_state_get_by_name(context, osd):
    return IMPL.osd_state_get_by_name(context, osd)

def osd_state_update(context, id, values):
    return IMPL.osd_state_update(context, id, values)

def osd_state_update_or_create(context, values):
    return IMPL.osd_state_update_or_create(context, values)

def osd_state_count_by_init_node_id(context, init_node_id):
    return IMPL.osd_state_count_by_init_node_id(context, init_node_id)

def osd_state_get_by_service_id_and_storage_group_id(context, service_id,\
                                                     storage_group_id):
    return IMPL.osd_state_get_by_service_id_and_storage_group_id(context,\
                                               service_id, storage_group_id)

def osd_state_get_by_device_id_and_service_id_and_cluster_id(\
        context, device_id, service_id, cluster_id):
    return IMPL.osd_state_get_by_device_id_and_service_id_and_cluster_id(\
        context, device_id, service_id, cluster_id)

def update_deleted_osd_state_by_service_id(context, service_id, values):
    return IMPL.update_deleted_osd_state_by_service_id(context, service_id, values)

def get_zone_hostname_storagegroup_by_osd_id(context, osd_id):
    return IMPL.get_zone_hostname_storagegroup_by_osd_id(context, osd_id)

def osd_state_get_by_service_id(context, service_id):
    return IMPL.osd_state_get_by_service_id(context, service_id)

def osd_state_get_by_osd_name_and_service_id_and_cluster_id(\
        context, osd_name, service_id, cluster_id):
    return IMPL.osd_state_get_by_osd_name_and_service_id_and_cluster_id(\
        context, osd_name, service_id, cluster_id)

#storage_group
def storage_group_get_all(context):
    return IMPL.storage_group_get_all(context)

def osd_state_count_by_storage_group_id(context, storage_group_id):
    return IMPL.osd_state_count_by_storage_group_id(context, storage_group_id)

def osd_state_count_service_id_by_storage_group_id(context, storage_group_id):
    return IMPL.osd_state_count_service_id_by_storage_group_id(context, \
                storage_group_id)

#zone 
def zone_get_all(context):
    return IMPL.zone_get_all(context)

def zone_get_by_id(context, id):
    return IMPL.zone_get_by_id(context,id)

def zone_get_by_name(context, zone):
    return IMPL.zone_get_by_name(context, zone)

def zone_get_all_in_crush(context):
    return IMPL.zone_get_all_in_crush(context)

def zone_get_all_not_in_crush(context):
    return IMPL.zone_get_all_not_in_crush(context)

#device

def device_get_all(context):
    return IMPL.device_get_all(context)

def device_get_count(context):
    return IMPL.device_get_count(context)

def device_get_by_service_id(context, service_id):
    return IMPL.device_get_by_service_id(context, service_id)

def device_get_distinct_storage_class_by_service_id(context, service_id):
    return IMPL.device_get_distinct_storage_class_by_service_id(context,\
            service_id)

def device_update_or_create(context, values):
    return IMPL.device_update_or_create(context, values)

def device_get_by_name_and_journal_and_service_id(context, name, \
                                                journal, service_id):
    return IMPL.device_get_by_name_and_journal_and_service_id(context, \
                                            name, journal, service_id)


def zone_update_or_create(context, values):
    return IMPL.zone_update_or_create(context, values)

def storage_group_update_or_create(context, values):
    return IMPL.storage_group_update_or_create(context, values)

#region vsmapp db APIs
def appnodes_get_by_id(context, id):
    return IMPL.appnodes_get_by_id(context, id)

def appnodes_get_all(context):
    return IMPL.appnodes_get_all(context)

def appnodes_create(context, values, allow_duplicate=False):
    return IMPL.appnodes_create(context, values, allow_duplicate)

def appnodes_update(context, appnode_id, values):
    return IMPL.appnodes_update(context, appnode_id, values)

def appnodes_destroy(context, appnode_id=None):
    return IMPL.appnodes_destroy(context, appnode_id)

def vsmapps_get_by_user(context):
    return IMPL.vsmapps_get_by_user(context)

def vsmapps_get_by_id(context, vsmapp_id):
    return IMPL.vsmapps_get_by_id(context, vsmapp_id)
#endregion

#region storage pool usage db APIs
def storage_pool_usage_create(context, pools):
    return IMPL.sp_usage_create(context, pools)

def storage_pool_usage_update(context, usage_id, values):
    return IMPL.sp_usage_update(context, usage_id, values)

def get_storage_pool_usage(context):
    return IMPL.get_sp_usage(context)

def get_sp_usage_by_poolid_vsmappid(context, pool_id, vsmapp_id):
    return IMPL.get_sp_usage_by_poolid_vsmappid(context, pool_id, vsmapp_id)

def destroy_storage_pool_usage(context, id):
    return IMPL.sp_usage_destroy(context, id)
#endregion

#region summary api
def summary_create(context, values):
    return IMPL.summary_create(context, values)

def summary_update(context, cluster_id, stype, values):
    return IMPL.summary_update(context, cluster_id, stype, values)

def summary_get_by_cluster_id_and_type(context, cluster_id, stype):
    return IMPL.summary_get_by_cluster_id_and_type(context, cluster_id, stype)

def summary_get_by_type_first(context, stype):
    return IMPL.summary_get_by_type_first(context, stype)
#endregion

#region monitor api

def monitor_get_all(context):
    return IMPL.monitor_get_all(context)

def monitor_get_by_address(context, monitor_address,read_deleted):
    return IMPL.monitor_get_by_address(context,monitor_address=monitor_address,
                                       read_deleted=read_deleted)

def monitor_get(context, id):
    return IMPL.monitor_get(context, id)

def monitor_update(context, mon_name, values):
    return IMPL.monitor_update(context, mon_name, values)

def monitor_destroy(context, mon_name):
    IMPL.monitor_destroy(context, mon_name)

#endregion

#pg api
def pg_create(context, values):
    return IMPL.pg_create(context, values)

def pg_get_all(context, limit=None, marker=None, sort_keys=None,
               sort_dir=None):
    return IMPL.pg_get_all(context, limit=limit, marker=marker,
                           sort_keys=sort_keys, sort_dir=sort_dir)

def pg_get(context, id):
    return IMPL.pg_get(context, id)

def pg_get_by_pgid(context, pgid):
    return IMPL.pg_get_by_pgid(context, pgid)

def pg_update(context, pg_id, values):
    return IMPL.pg_update(context, pg_id, values)

def pg_update_or_create(context, values):
    return IMPL.pg_update_or_create(context, values)

#rbd
def rbd_create(context, values):
    return IMPL.rbd_create(context, values)

def rbd_get_all(context, limit=None, marker=None, sort_keys=None, sort_dir=None):
    return IMPL.rbd_get_all(context, limit=limit, marker=marker, sort_keys=sort_keys,
							sort_dir=sort_dir)

def rbd_get(context, id):
    return IMPL.rbd_get(context, id)

def rbd_get_by_pool_and_image(context, pool, image):
    return IMPL.rbd_get_by_pool_and_image(context, pool, image)

def rbd_update(context, rbd_id, values):
    return IMPL.rbd_update(context, rbd_id, values)

def rbd_update_or_create(context, values):
    return IMPL.rbd_update_or_create(context, values)

#region license status ops
def license_status_create(context, values):
    return IMPL.license_status_create(context, values)

def license_status_get(context):
    return IMPL.license_status_get(context)

def license_status_update(context, value):
    return IMPL.license_status_update(context, value)

#endregion

#mds
def mds_create(context, values):
    return IMPL.mds_create(context, values)

def mds_get_all(context):
    return IMPL.mds_get_all(context)

def mds_get_by_id(context, id):
    return IMPL.mds_get(context, id)

def mds_get_by_gid(context, gid):
    return IMPL.mds_get_by_gid(context, gid)

def mds_update(context, mds_id, values):
    return IMPL.mds_update(context, mds_id, values)

def mds_update_or_create(context, values):
    return IMPL.mds_update_or_create(context, values)

#region vsm settings db api
def vsm_settings_update_or_create(context, values):
    return IMPL.vsm_settings_update_or_create(context, values)

def vsm_settings_get_all(context):
    return IMPL.vsm_settings_get_all(context)

def vsm_settings_get_by_name(context, name):
    return IMPL.vsm_settings_get_by_name(context, name)
#endregion

#long_call
def long_call_create(context, values):
    return IMPL.long_call_create(context, values)

def long_call_get_by_uuid(context, uuid):
    return IMPL.long_call_get_by_uuid(context, uuid)

def long_call_update(context, long_call_uuid, values):
    return IMPL.long_call_update(context, long_call_uuid, values)

def long_call_delete(context, long_call_uuid):
    return IMPL.long_call_delete(context, long_call_uuid)
#end

#region ec profile db api
def ec_profile_update_or_create(context, values):
    return IMPL.ec_profile_update_or_create(context, values)

def ec_profile_get_all(context):
    return IMPL.ec_profile_get_all(context)

def ec_profile_get(context, ec_profile_id):
    return IMPL.ec_profile_get(context, ec_profile_id)

def ec_profile_get_by_name(context, name):
    return IMPL.ec_profile_get_by_name(context, name)

def ec_profile_remove(context, ec_profile_id):
    return IMPL.ec_profile_remove(context, ec_profile_id)

def get_performance_metrics(context, search_opts):
    return IMPL.performance_metrics_query(context, search_opts=search_opts)

def get_sum_performance_metrics(context, search_opts):
    return IMPL.sum_performance_metrics(context, search_opts=search_opts)

def get_latency(context, search_opts):
    return IMPL.latency_performance_metrics(context, search_opts=search_opts)

def get_cpu_usage(context, search_opts):
    return IMPL.cpu_data_get_usage(context, search_opts=search_opts)

def get_latency_all_types(context, search_opts):
    return IMPL.latency_all_types(context, search_opts=search_opts)

def get_iops_all_types(context, search_opts):
    return IMPL.iops_all_type(context, search_opts=search_opts)

def get_bandwidth_all_types(context, search_opts):
    return IMPL.bandwidth_all_type(context, search_opts=search_opts)

def clean_performance_history_data(context,days):
    return IMPL.clean_performance_history_data(context,days)

def get_poolusage(context, poolusage_id):
    return IMPL.get_poolusage(context, poolusage_id=poolusage_id)
#endregion



# vim: tabstop=4 shiftwidth=4 softtabstop=4

#    Copyright 2013 Cloudscaling Group, Inc
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
The MatchMaker classes should accept a Topic or Fanout exchange key and
return keys for direct exchanges, per (approximate) AMQP parlance.
"""

from oslo.config import cfg

from vsm.openstack.common import importutils
from vsm.openstack.common import log as logging
from vsm.openstack.common.rpc import matchmaker as mm_common

redis = importutils.try_import('redis')

matchmaker_redis_opts = [
    cfg.StrOpt('host',
               default='127.0.0.1',
               help='Host to locate redis'),
    cfg.IntOpt('port',
               default=6379,
               help='Use this port to connect to redis host.'),
    cfg.StrOpt('password',
               default=None,
               help='Password for Redis server. (optional)'),
]

CONF = cfg.CONF
opt_group = cfg.OptGroup(name='matchmaker_redis',
                         title='Options for Redis-based MatchMaker')
CONF.register_group(opt_group)
CONF.register_opts(matchmaker_redis_opts, opt_group)
LOG = logging.getLogger(__name__)

class RedisExchange(mm_common.Exchange):
    def __init__(self, matchmaker):
        self.matchmaker = matchmaker
        self.redis = matchmaker.redis
        super(RedisExchange, self).__init__()

class RedisTopicExchange(RedisExchange):
    """
    Exchange where all topic keys are split, sending to second half.
    i.e. "compute.host" sends a message to "compute" running on "host"
    """
    def run(self, topic):
        while True:
            member_name = self.redis.srandmember(topic)

            if not member_name:
                # If this happens, there are no
                # longer any members.
                break

            if not self.matchmaker.is_alive(topic, member_name):
                continue

            host = member_name.split('.', 1)[1]
            return [(member_name, host)]
        return []

class RedisFanoutExchange(RedisExchange):
    """
    Return a list of all hosts.
    """
    def run(self, topic):
        topic = topic.split('~', 1)[1]
        hosts = self.redis.smembers(topic)
        good_hosts = filter(
            lambda host: self.matchmaker.is_alive(topic, host), hosts)

        return [(x, x.split('.', 1)[1]) for x in good_hosts]

class MatchMakerRedis(mm_common.HeartbeatMatchMakerBase):
    """
    MatchMaker registering and looking-up hosts with a Redis server.
    """
    def __init__(self):
        super(MatchMakerRedis, self).__init__()

        if not redis:
            raise ImportError("Failed to import module redis.")

        self.redis = redis.StrictRedis(
            host=CONF.matchmaker_redis.host,
            port=CONF.matchmaker_redis.port,
            password=CONF.matchmaker_redis.password)

        self.add_binding(mm_common.FanoutBinding(), RedisFanoutExchange(self))
        self.add_binding(mm_common.DirectBinding(), mm_common.DirectExchange())
        self.add_binding(mm_common.TopicBinding(), RedisTopicExchange(self))

    def ack_alive(self, key, host):
        topic = "%s.%s" % (key, host)
        if not self.redis.expire(topic, CONF.matchmaker_heartbeat_ttl):
            # If we could not update the expiration, the key
            # might have been pruned. Re-register, creating a new
            # key in Redis.
            self.register(self.topic_host[host], host)

    def is_alive(self, topic, host):
        if self.redis.ttl(host) == -1:
            self.expire(topic, host)
            return False
        return True

    def expire(self, topic, host):
        with self.redis.pipeline() as pipe:
            pipe.multi()
            pipe.delete(host)
            pipe.srem(topic, host)
            pipe.execute()

    def backend_register(self, key, key_host):
        with self.redis.pipeline() as pipe:
            pipe.multi()
            pipe.sadd(key, key_host)

            # No value is needed, we just
            # care if it exists. Sets aren't viable
            # because only keys can expire.
            pipe.set(key_host, '')

            pipe.execute()

    def backend_unregister(self, key, key_host):
        with self.redis.pipeline() as pipe:
            pipe.multi()
            pipe.srem(key, key_host)
            pipe.delete(key_host)
            pipe.execute()




# Copyright 2014 Intel Inc.
#All Rights Reserved.

# Licensed under the Apache License, Version 2.0 (the"License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at

#  http://www.apache.org/licenses/LICENSE-2.0

# Unless required by applicable law or agreed to in writing,
# software distributed under the License is distributed on an
# "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
# KIND, either express or implied. See the License for the
# specific language governing permissions and limitations
# under the License.

# -*- encoding: utf-8 -*-
#
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from migrate import ForeignKeyConstraint
from sqlalchemy import MetaData, Table
from sqlalchemy.sql.expression import select

from vsm.db.sqlalchemy import utils

def upgrade(migrate_engine):
    if migrate_engine.name == 'sqlite':
        return
    meta = MetaData(bind=migrate_engine)
    storage_pools = Table('storage_pools', meta, autoload=True)
    storage_groups = Table('storage_groups', meta, autoload=True)
    params = {'columns': [storage_pools.c.primary_storage_group_id],
              'refcolumns': [storage_groups.c.id]}
    if migrate_engine.name == 'mysql':
        params['name'] = "_".join(('storage_pool', 'primary_storage_group_id', 'fkey'))
    fkey = ForeignKeyConstraint(**params)
    fkey.create()

def downgrade(migrate_engine):
    if migrate_engine.name == 'sqlite':
        return
    meta = MetaData(bind=migrate_engine)
    storage_pools = Table('storage_pools', meta, autoload=True)
    storage_groups = Table('storage_groups', meta, autolaod=True)
    params = {'columns': [storage_pools.c.primary_storage_group_id],
              'refcolumns': [storage_groups.c.id]}
    fkey = ForeignKeyConstraint(**params)
    fkey.drop()                        



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2010 OpenStack LLC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import os
import re
import urlparse

import webob

from vsm.api.openstack import wsgi
from vsm.api import xmlutil
from vsm import flags
from vsm.openstack.common import log as logging
from vsm import utils

LOG = logging.getLogger(__name__)
FLAGS = flags.FLAGS

XML_NS_V1 = 'http://docs.openstack.org/storage/api/v1'

def get_pagination_params(request):
    """Return marker, limit tuple from request.

    :param request: `wsgi.Request` possibly containing 'marker' and 'limit'
                    GET variables. 'marker' is the id of the last element
                    the client has seen, and 'limit' is the maximum number
                    of items to return. If 'limit' is not specified, 0, or
                    > max_limit, we default to max_limit. Negative values
                    for either marker or limit will cause
                    exc.HTTPBadRequest() exceptions to be raised.

    """
    params = {}
    if 'limit' in request.GET:
        params['limit'] = _get_limit_param(request)
    if 'marker' in request.GET:
        params['marker'] = _get_marker_param(request)
    return params

def _get_limit_param(request):
    """Extract integer limit from request or fail"""
    try:
        limit = int(request.GET['limit'])
    except ValueError:
        msg = _('limit param must be an integer')
        raise webob.exc.HTTPBadRequest(explanation=msg)
    if limit < 0:
        msg = _('limit param must be positive')
        raise webob.exc.HTTPBadRequest(explanation=msg)
    return limit

def _get_marker_param(request):
    """Extract marker id from request or fail"""
    return request.GET['marker']

def limited(items, request, max_limit=FLAGS.osapi_max_limit):
    """Return a slice of items according to requested offset and limit.

    :param items: A sliceable entity
    :param request: ``wsgi.Request`` possibly containing 'offset' and 'limit'
                    GET variables. 'offset' is where to start in the list,
                    and 'limit' is the maximum number of items to return. If
                    'limit' is not specified, 0, or > max_limit, we default
                    to max_limit. Negative values for either offset or limit
                    will cause exc.HTTPBadRequest() exceptions to be raised.
    :kwarg max_limit: The maximum number of items to return from 'items'
    """
    try:
        offset = int(request.GET.get('offset', 0))
    except ValueError:
        msg = _('offset param must be an integer')
        raise webob.exc.HTTPBadRequest(explanation=msg)

    try:
        limit = int(request.GET.get('limit', max_limit))
    except ValueError:
        msg = _('limit param must be an integer')
        raise webob.exc.HTTPBadRequest(explanation=msg)

    if limit < 0:
        msg = _('limit param must be positive')
        raise webob.exc.HTTPBadRequest(explanation=msg)

    if offset < 0:
        msg = _('offset param must be positive')
        raise webob.exc.HTTPBadRequest(explanation=msg)

    limit = min(max_limit, limit or max_limit)
    range_end = offset + limit
    return items[offset:range_end]

def limited_by_marker(items, request, max_limit=FLAGS.osapi_max_limit):
    """Return a slice of items according to the requested marker and limit."""
    params = get_pagination_params(request)

    limit = params.get('limit', max_limit)
    marker = params.get('marker')

    limit = min(max_limit, limit)
    start_index = 0
    if marker:
        start_index = -1
        for i, item in enumerate(items):
            if 'flavorid' in item:
                if item['flavorid'] == marker:
                    start_index = i + 1
                    break
            elif item['id'] == marker or item.get('uuid') == marker:
                start_index = i + 1
                break
        if start_index < 0:
            msg = _('marker [%s] not found') % marker
            raise webob.exc.HTTPBadRequest(explanation=msg)
    range_end = start_index + limit
    return items[start_index:range_end]

def remove_version_from_href(href):
    """Removes the first api version from the href.

    Given: 'http://www.vsm.com/v1.1/123'
    Returns: 'http://www.vsm.com/123'

    Given: 'http://www.vsm.com/v1.1'
    Returns: 'http://www.vsm.com'

    """
    parsed_url = urlparse.urlsplit(href)
    url_parts = parsed_url.path.split('/', 2)

    # NOTE: this should match vX.X or vX
    expression = re.compile(r'^v([0-9]+|[0-9]+\.[0-9]+)(/.*|$)')
    if expression.match(url_parts[1]):
        del url_parts[1]

    new_path = '/'.join(url_parts)

    if new_path == parsed_url.path:
        msg = _('href %s does not contain version') % href
        LOG.debug(msg)
        raise ValueError(msg)

    parsed_url = list(parsed_url)
    parsed_url[2] = new_path
    return urlparse.urlunsplit(parsed_url)

def dict_to_query_str(params):
    # TODO(throughnothing): we should just use urllib.urlencode instead of this
    # But currently we don't work with urlencoded url's
    param_str = ""
    for key, val in params.iteritems():
        param_str = param_str + '='.join([str(key), str(val)]) + '&'

    return param_str.rstrip('&')

class ViewBuilder(object):
    """Model API responses as dictionaries."""

    _collection_name = None

    def _get_links(self, request, identifier):
        return [{"rel": "self",
                 "href": self._get_href_link(request, identifier), },
                {"rel": "bookmark",
                 "href": self._get_bookmark_link(request, identifier), }]

    def _get_next_link(self, request, identifier):
        """Return href string with proper limit and marker params."""
        params = request.params.copy()
        params["marker"] = identifier
        prefix = self._update_link_prefix(request.application_url,
                                          FLAGS.vsmapi_storage_base_URL)
        url = os.path.join(prefix,
                           request.environ["vsm.context"].project_id,
                           self._collection_name)
        return "%s?%s" % (url, dict_to_query_str(params))

    def _get_href_link(self, request, identifier):
        """Return an href string pointing to this object."""
        prefix = self._update_link_prefix(request.application_url,
                                          FLAGS.vsmapi_storage_base_URL)
        return os.path.join(prefix,
                            request.environ["vsm.context"].project_id,
                            self._collection_name,
                            str(identifier))

    def _get_bookmark_link(self, request, identifier):
        """Create a URL that refers to a specific resource."""
        base_url = remove_version_from_href(request.application_url)
        base_url = self._update_link_prefix(base_url,
                                            FLAGS.vsmapi_storage_base_URL)
        return os.path.join(base_url,
                            request.environ["vsm.context"].project_id,
                            self._collection_name,
                            str(identifier))

    def _get_collection_links(self, request, items, id_key="uuid"):
        """Retrieve 'next' link, if applicable."""
        links = []
        limit = int(request.params.get("limit", 0))
        if limit and limit == len(items):
            last_item = items[-1]
            if id_key in last_item:
                last_item_id = last_item[id_key]
            else:
                last_item_id = last_item["id"]
            links.append({
                "rel": "next",
                "href": self._get_next_link(request, last_item_id),
            })
        return links

    def _update_link_prefix(self, orig_url, prefix):
        if not prefix:
            return orig_url
        url_parts = list(urlparse.urlsplit(orig_url))
        prefix_parts = list(urlparse.urlsplit(prefix))
        url_parts[0:2] = prefix_parts[0:2]
        return urlparse.urlunsplit(url_parts)

class MetadataDeserializer(wsgi.MetadataXMLDeserializer):
    def deserialize(self, text):
        dom = utils.safe_minidom_parse_string(text)
        metadata_node = self.find_first_child_named(dom, "metadata")
        metadata = self.extract_metadata(metadata_node)
        return {'body': {'metadata': metadata}}

class MetaItemDeserializer(wsgi.MetadataXMLDeserializer):
    def deserialize(self, text):
        dom = utils.safe_minidom_parse_string(text)
        metadata_item = self.extract_metadata(dom)
        return {'body': {'meta': metadata_item}}

class MetadataXMLDeserializer(wsgi.XMLDeserializer):

    def extract_metadata(self, metadata_node):
        """Marshal the metadata attribute of a parsed request"""
        if metadata_node is None:
            return {}
        metadata = {}
        for meta_node in self.find_children_named(metadata_node, "meta"):
            key = meta_node.getAttribute("key")
            metadata[key] = self.extract_text(meta_node)
        return metadata

    def _extract_metadata_container(self, datastring):
        dom = utils.safe_minidom_parse_string(datastring)
        metadata_node = self.find_first_child_named(dom, "metadata")
        metadata = self.extract_metadata(metadata_node)
        return {'body': {'metadata': metadata}}

    def create(self, datastring):
        return self._extract_metadata_container(datastring)

    def update_all(self, datastring):
        return self._extract_metadata_container(datastring)

    def update(self, datastring):
        dom = utils.safe_minidom_parse_string(datastring)
        metadata_item = self.extract_metadata(dom)
        return {'body': {'meta': metadata_item}}

metadata_nsmap = {None: xmlutil.XMLNS_V11}

class MetaItemTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        sel = xmlutil.Selector('meta', xmlutil.get_items, 0)
        root = xmlutil.TemplateElement('meta', selector=sel)
        root.set('key', 0)
        root.text = 1
        return xmlutil.MasterTemplate(root, 1, nsmap=metadata_nsmap)

class MetadataTemplateElement(xmlutil.TemplateElement):
    def will_render(self, datum):
        return True

class MetadataTemplate(xmlutil.TemplateBuilder):
    def construct(self):
        root = MetadataTemplateElement('metadata', selector='metadata')
        elem = xmlutil.SubTemplateElement(root, 'meta',
                                          selector=xmlutil.get_items)
        elem.set('key', 0)
        elem.text = 1
        return xmlutil.MasterTemplate(root, 1, nsmap=metadata_nsmap)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Storage pool usage operations.
"""

from vsm import context
from vsm import db
from vsm import utils
from vsm import exception
from vsm.openstack.common import timeutils
from vsm.openstack.common.gettextutils import _
from vsm.openstack.common import log as logging
from vsm.openstack.common.db import exception as db_exc

LOG = logging.getLogger(__name__)

def get_all(contxt):
    """get all non-deleted storage pool usage as a dict"""
    if contxt is None:
        contxt = context.get_admin_context()
    try:
        uses = db.get_storage_pool_usage(contxt)
        return uses
    except db_exc.DBError as e:
        LOG.exception(_("DB Error on getting Storage Pool Usage %s" % e))
        raise exception.StoragePoolUsageFailure()

def create(contxt, pool_list):
    """create storage pool usages"""
    return db.storage_pool_usage_create(contxt, pool_list)

def update(contxt, vsmapp_id, attach_status=None, is_terminate=False):
    """update storage pool usage"""
    if contxt is None:
        contxt = context.get_admin_context()

    if not vsmapp_id:
        raise exception.StoragePoolUsageInvalid()

    is_terminate = utils.bool_from_str(is_terminate)

    kargs = {
        'attach_status': attach_status,
        'terminate_at': timeutils.utcnow() if is_terminate else None
    }

    try:
        return db.storage_pool_usage_update(contxt, vsmapp_id, kargs)
    except db_exc.DBError as e:
        LOG.exception(_("DB Error on updating new storage pool usage %s" % e))
        raise exception.StoragePoolUsageFailure()

def destroy(contxt, id):
    if contxt is None:
        contxt = context.get_admin_context()

    id = utils.int_from_str(id)
    try:
        db.destroy_storage_pool_usage(contxt, id)
    except db_exc.DBError as e:
        LOG.exception(_("DB Error on deleting Pool Usages %s" % e))
        raise exception.StoragePoolUsageFailure()




# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2012 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
gettext for openstack-common modules.

Usual usage in an openstack.common module:

    from vsm.openstack.common.gettextutils import _
"""

import gettext

t = gettext.translation('openstack-common', 'locale', fallback=True)

def _(msg):
    return t.ugettext(msg)



#!/usr/bin/env python
# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Starter script for Vsm Hardware."""

import eventlet
eventlet.monkey_patch()

import os
import sys
from oslo.config import cfg

possible_topdir = os.path.normpath(os.path.join(os.path.abspath(sys.argv[0]),
                                   os.pardir,
                                   os.pardir))
if os.path.exists(os.path.join(possible_topdir, 'vsm', '__init__.py')):
    sys.path.insert(0, possible_topdir)

from vsm import flags
from vsm.openstack.common import log as logging
from vsm import service
from vsm import utils

FLAGS = flags.FLAGS

if __name__ == '__main__':
    flags.parse_args(sys.argv)
    logging.setup("vsm")
    utils.monkey_patch()
    launcher = service.ProcessLauncher()
    server = service.Service.create(binary='vsm-testdb')
    launcher.launch_server(server)
    launcher.wait()



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""
Drivers for testdbs.

"""

import os
import socket
import time
import re
import string
import atexit

from oslo.config import cfg
from vsm import db
from vsm import exception
from vsm.image import image_utils
from vsm.openstack.common import log as logging
from vsm import utils
from vsm.conductor import rpcapi as conductor_rpcapi

LOG = logging.getLogger(__name__)

CONF = cfg.CONF

class TestDBDriver(object):
    """Executes commands relating to TestDBs."""
    def __init__(self, execute=utils.execute, *args, **kwargs):
        pass

    def init_host(self, host):
        pass

    def service_get_all(self, context):
        service_list = db.service_get_all(context)
        for x in service_list:
            LOG.debug('x.id = %s' % x.id)
            LOG.debug('x.topic = %s' % x.topic)
            LOG.debug('x.recipeId = %s' % x.recipe_Id)

class TestPoolDriver(object):
    """Executes commands relating to pool table."""
    def __init__(self, *args, **kwargs):
        pass

    def storage_pool_get_all(self, context):
        pool_list = db.pool_get_all(context)
        LOG.info('Pools: %s' % pool_list)
        for x in pool_list:
            LOG.debug('x.id = %s' % x.id)
            LOG.debug('x.name = %s' % x.name)
            LOG.debug('x.ruleset = %s' % x.ruleset)
        return pool_list

    def storage_pool_update(self, context, pool_id, values):
        db.pool_update(context, pool_id, values)

class TestOsdDriver(object):
    """Execute command relation with Osd table."""
    def __init__(self, execute=utils.execute, *args, **kwargs):
        pass

    def init_host(self, host):
        pass
    
    def osd_get_all(self, context):
        LOG.debug("DEBUG test osd_get_all func")
        osd_list = db.osd_state_get_all(context)
        len = osd_list.__len__()
        LOG.debug(len)
        for x in osd_list:
            LOG.debug('x.id = %s' % x.id)
            LOG.debug('x.osd_name = %s' % x.osd_name)
            LOG.debug('x.state = %s' % x.state)
            LOG.debug('x.service_id = %s' % x.service_id)
            LOG.debug('x.device_id = %s' % x.device_id)
            LOG.debug('x.device.total_capacity = %s' % x.device.total_capacity_kb)
        return osd_list
        
    def osd_get_all_down(self, context):
        LOG.debug("DEBUG test osd_get_all_down func")
        osd_list = db.osd_get_all_down(context)
        for x in osd_list:
            LOG.debug('x.id = %s' % x.id)
            LOG.debug('x.osd_name = %s' % x.osd_name)
            LOG.debug('x.state = %s' % x.state)
            LOG.debug('x.service_id = %s' % x.service_id)
            LOG.debug('x.device_id = %s' % x.device_id)
    
    def osd_get_all_up(self, context):
        LOG.debug("DEBUG test osd_get_all_down func")
        osd_list = db.osd_get_all_up(context)
        for x in osd_list:
            LOG.debug('x.id = %s' % x.id)
            LOG.debug('x.osd_name = %s' % x.osd_name)
            LOG.debug('x.state = %s' % x.state)
            LOG.debug('x.service_id = %s' % x.service_id)
            LOG.debug('x.device_id = %s' % x.device_id)
    
    def osd_get_by_service_id(self, context, service_id):
        LOG.debug("DEBUG test osd_get_by_service_id func")
        osd_list = db.osd_get_by_service_id(context, service_id)
        for x in osd_list:
            LOG.debug('x.id = %s' % x.id)
            LOG.debug('x.osd_name = %s' % x.osd_name)
            LOG.debug('x.state = %s' % x.state)
            LOG.debug('x.service_id = %s' % x.service_id)
            LOG.debug('x.device_id = %s' % x.device_id)
    
    def osd_get_by_cluster_id(self, context, cluster_id):
        LOG.debug("DEBUG test osd_get_by_cluster_id func")
        osd_list = db.osd_get_by_cluster_id(context, cluster_id)
        for x in osd_list:
            LOG.debug('x.id = %s' % x.id)
            LOG.debug('x.osd_name = %s' % x.osd_name)
            LOG.debug('x.state = %s' % x.state)
            LOG.debug('x.service_id = %s' % x.service_id)
            LOG.debug('x.device_id = %s' % x.device_id)
 
    def osd_get(self, context, id):
        LOG.debug("DEBUG test osd_get func")
        osd_list = db.osd_get(context, id)
        #len = osd_list.__len__()
        #LOG.debug(len)
        print osd_list
        for x in osd_list:
            print(x)
    
    def osd_destroy(self, context):
        LOG.debug("DEBUG test osd_destroy func")
        db.osd_destroy(context)
 
    def osd_create(self, context, values):
        LOG.debug("DEBUG test osd_create func")
        db.osd_create(context, values)

    def osd_update(self, context, osd_id, values):
        LOG.debug("DEBUG test osd_update func")
        db.osd_update(context, osd_id, values)

    def osd_delete(self, context, osd_id):
        LOG.debug("DEBUG test osd_delete func")
        db.osd_delete(context, osd_id)

class TestCrushMapDriver(object):
    """Execute command relation with CRUSH map table."""
    def __init__(self, execute=utils.execute, *args, **kwargs):
        pass

    def init_host(self, host):
        pass
    
    def crushmap_create(self, context, values):
        LOG.debug("DEBUG test crushmap_create func")
        db.crushmap_create(context, values)

    def crushmap_get_all(self, context):
        LOG.debug("DEBUG test crushmap_get_all func")
        crushmap_list = db.crushmap_get_all(context)
        for x in crushmap_list:
            LOG.debug("crushmap id = %s" % x.id)
            LOG.debug("crushmap content = %s" % x.content)
            LOG.debug("crushmap created_at = %s" % x.created_at)
            LOG.debug("crushmap deleted = %s" % x.deleted)
    
    def crushmap_get(self, context, crushmap_id):
        LOG.debug("DEBUG test crushmap_get func")
        crushmap_list = db.crushmap_get(context, crushmap_id)
        for x in crushmap_list:
            print x

    def crushmap_update(self, context, crushmap_id, values):
        LOG.debug("DEBUG test crushmap_update func")
        db.crushmap_update(context, crushmap_id, values)

    def crushmap_delete(self, context, crushmap_id):
        LOG.debug("DEBUG test crushmap_delete func")
        db.crushmap_delete(context, crushmap_id)

class TestServiceDriver(object):
    """Execute command relation with services table."""
    def __init__(self, execute=utils.execute, *args, **kwargs):
        pass

    def init_host(self, host):
        pass

    def service_create(self, context, values):
        LOG.debug("DEBUG test service_create func")
        db.service_create(context, values)

    def service_get_all(self, context, disabled=None):
        LOG.debug("DEBUG test service_get_all func")
        service_list = db.service_get_all(context)
        for x in service_list:
            LOG.debug("service id = %s" % x.id)
            LOG.debug("service host = %s" % x.host)
            LOG.debug("service binary = %s" % x.binary)
            LOG.debug("service topic = %s" % x.topic)
            LOG.debug("service report_count = %s" % x.report_count)
            LOG.debug("service created_at = %s" % x.created_at)

    def service_get(self, context, service_id):
        LOG.debug("DEBUG test service_get func")
        service_list = db.service_get(context, service_id)
        for x in service_list:
            print(x)

    def service_update(self, context, service_id, values):
        LOG.debug("DEBUG test service_update func")
        db.service_update(context, service_id, values)

class TestDeviceDriver(object):
    """Execute command relation with devices table."""
    def __init__(self, execute=utils.execute, *args, **kwargs):
        pass

    def init_host(self, host):
        pass
    
    def device_create(self, context, values):
        LOG.debug("DEBUG test device_create func")
        device_list = db.device_create(context, values)
    
    def device_get_all_by_interface_type(self, context, interface_type):
        LOG.debug("DEBUG test device_get_all_by_interface_type func")
        device_list = db.device_get_all_by_interface_type(context, interface_type)
        print(device_list)
        for x in device_list:
            LOG.debug("device id = %s" % x.id)
            LOG.debug("device name = %s" % x.name)
            LOG.debug("service id = %s" % x.service_id)
            LOG.debug("device device type = %s" % x.device_type)
            LOG.debug("device interface type = %s" % x.interface_type)
            LOG.debug("device device state = %s" % x.state)
    
    def device_get_all_by_device_type(self, context, device_type):
        LOG.debug("DEBUG test device_get_all_by_device_type func")
        device_list = db.device_get_all_by_device_type(context, device_type)
        for x in device_list:
            LOG.debug("device id = %s" % x.id)
            LOG.debug("device name = %s" % x.name)
            LOG.debug("service id = %s" % x.service_id)
            LOG.debug("device device type = %s" % x.device_type)
            LOG.debug("device interface type = %s" % x.interface_type)
            LOG.debug("device device state = %s" % x.state)

class TestSummaryDriver(object):
    """ create and update summary table"""
    def __init__(self, *args, **kwargs):
        pass

    def create_summary(self, context, values):
        LOG.debug('create summary. ')
        summary = db.summary_create(context, values)
        return summary

    def update_summary(self, context, cluster_id, typ, values):
        LOG.debug('update summary. ')
        summary = db.summary_update(context, cluster_id, typ, values)
        return summary

    def get_summary_by_id_and_type(self, context, cluster_id, typ):
        LOG.debug('get summary. ')
        summary = db.summary_get_by_cluster_id_and_type(context, cluster_id, typ)
        return summary

class TestMonDriver(object):
    """ create and update monitors table"""
    def __init__(self, *args, **kwargs):
        pass

    def get_all_monitors(self, context):
        return db.monitor_get_all(context)

    def update_monitor(self, context, mon_name, values):
        LOG.debug('create/update monitor. ')
        mon = db.monitor_update(context, mon_name, values)
        return mon

class TestPGDriver(object):
    def __init__(self, *args, **kwargs):
        self._conductor_rpcapi = conductor_rpcapi.ConductorAPI()

    def pg_create(self, context, values):
        return db.pg_create(context, values)

    def pg_update(self, context, pg_id, values):
        return db.pg_update(context, pg_id, values)

    def pg_get_all(self, context):
        return self._conductor_rpcapi.pg_get_all(context)

    def pg_update_or_create(self, context, values):
        return self._conductor_rpcapi.pg_update_or_create(context, values)

class TestRbdDriver(object):
    def __init__(self, *args, **kwargs):
        self._conductor_rpcapi = conductor_rpcapi.ConductorAPI()

    def rbd_create(self, context, values):
        return db.rbd_create(context, values)

    def rbd_update(self, context, rbd_id, values):
        return db.rbd_update(context, rbd_id, values)

    def rbd_get_all(self, context):
        return db.rbd_get_all(context)

    def rbd_update_or_create(self, context, values):
        return db.rbd_update_or_create(context, values)

class TestSettingDriver(object):
    def __init__(self):
        pass

    def setting_create(self, context, values):
        return db.vsm_settings_update_or_create(context, values)

    def setting_update(self, context, values):
        return db.vsm_settings_update_or_create(context, values)

    def settings_get_all(self, context):
        return db.vsm_settings_get_all(context)

    def settings_get_by_name(self, context, name):
        return db.vsm_settings_get_by_name(context, name)



# vim: tabstop=4 shiftwidth=4 softtabstop=4

# Copyright 2014 Intel Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from vsm.api import common
import logging
import time

LOG = logging.getLogger(__name__)

class ViewBuilder(common.ViewBuilder):
    _collection_name = "storage_groups"

    def basic(self, request, storage_group):
        LOG.info("storage_groups api view %s " % storage_group)
        _storage_group = {
            "storage_group": {
                "id": storage_group["id"],
                "name": storage_group["name"],
                "friendly_name": storage_group["friendly_name"].replace("_", " "),
                "storage_class": storage_group["storage_class"],
                "attached_pools": storage_group["attached_pools"],
                "attached_osds": storage_group["attached_osds"],
                "capacity_total": storage_group["capacity_total"],
                "capacity_used": storage_group["capacity_used"],
                "capacity_avail": storage_group["capacity_avail"],
                #"capacity_percent_used": "health",
                "largest_node_capacity_used": storage_group["largest_node_capacity_used"],
                "status": storage_group["status"],
                "take_list":storage_group["take_list"],
                "rule_id":storage_group["rule_id"],
                "marker": storage_group["marker"],

            }
        }
        try:
            _storage_group['storage_group']['updated_at'] = time.strftime("%Y-%m-%d %H:%M:%S", time.strptime(storage_group['updated_at'], "%Y-%m-%dT%H:%M:%S.000000"))
        except:
            _storage_group['storage_group']['updated_at'] = ""
        return _storage_group

    def index(self, request, storage_groups):
        """Show a list of storage_groups without many details."""
        return self._list_view(self.basic, request, storage_groups)

    def _list_view(self, func, request, storage_groups):
        """Provide a view for a list of storage_groups."""
        storage_group_list = [func(request, storage_group)["storage_group"] for storage_group in storage_groups]
        storage_groups_dict = dict(storage_groups=storage_group_list)
        return storage_groups_dict

